@@ -43,7 +43,7 @@ enum class LineReflowStatus {
 
 class nsBlockInFlowLineIterator;
 namespace mozilla {
-class BlockReflowInput;
+class BlockReflowState;
 class PresShell;
 class ServoRestyleState;
 class ServoStyleSet;
@@ -79,7 +79,7 @@ class ServoStyleSet;
  * contains the absolutely positioned frames.
  */
 class nsBlockFrame : public nsContainerFrame {
-  using BlockReflowInput = mozilla::BlockReflowInput;
+  using BlockReflowState = mozilla::BlockReflowState;
 
  public:
   NS_DECL_FRAMEARENA_HELPERS(nsBlockFrame)
@@ -175,61 +175,60 @@ class nsBlockFrame : public nsContainerFrame {
   mozilla::a11y::AccType AccessibleType() override;
 #endif
 
-  // Line cursor methods to speed up line searching in which one query
-  // result is expected to be close to the next in general. This is
-  // mainly for searching line(s) containing a point. It is also used
-  // as a cache for local computation. Use AutoLineCursorSetup for the
-  // latter case so that it wouldn't interact unexpectedly with the
-  // former. The basic idea for the former is that we set the cursor
-  // property if the lines' overflowArea.VisualOverflow().ys and
-  // overflowArea.VisualOverflow().yMosts are non-decreasing
-  // (considering only non-empty overflowArea.VisualOverflow()s; empty
-  // overflowArea.VisualOverflow()s never participate in event handling
+  // Line cursor methods to speed up line searching in which one query result
+  // is expected to be close to the next in general. This is mainly for
+  // searching line(s) containing a point. It is also used as a cache for local
+  // computation. The basic idea for the former is that we set the cursor
+  // property if the lines' overflowArea.InkOverflow().ys and
+  // overflowArea.InkOverflow().yMosts are non-decreasing
+  // (considering only non-empty overflowArea.InkOverflow()s; empty
+  // overflowArea.InkOverflow()s never participate in event handling
   // or painting), and the block has sufficient number of lines. The
   // cursor property points to a "recently used" line. If we get a
   // series of requests that work on lines
   // "near" the cursor, then we can find those nearby lines quickly by
   // starting our search at the cursor.
 
-  // Clear out line cursor because we're disturbing the lines (i.e., Reflow)
-  void ClearLineCursor();
+  // We have two independent line cursors, one used for display-list building
+  // and the other for a11y or other frame queries. Either or both may be
+  // present at any given time. When we reflow or otherwise munge the lines,
+  // both cursors will be cleared.
+  // The display cursor is only created and used if the lines satisfy the non-
+  // decreasing y-coordinate condition (see SetupLineCursorForDisplay comment
+  // below), whereas the query cursor may be created for any block. The two
+  // are separated so creating a cursor for a11y queries (eg GetRenderedText)
+  // does not risk confusing the display-list building code.
+
+  // Clear out line cursors because we're disturbing the lines (i.e., Reflow)
+  void ClearLineCursors() {
+    if (MaybeHasLineCursor()) {
+      ClearLineCursorForDisplay();
+      ClearLineCursorForQuery();
+      RemoveStateBits(NS_BLOCK_HAS_LINE_CURSOR);
+    }
+  }
+  void ClearLineCursorForDisplay() {
+    RemoveProperty(LineCursorPropertyDisplay());
+  }
+  void ClearLineCursorForQuery() { RemoveProperty(LineCursorPropertyQuery()); }
+
   // Get the first line that might contain y-coord 'y', or nullptr if you must
   // search all lines. If nonnull is returned then we guarantee that the lines'
   // combinedArea.ys and combinedArea.yMosts are non-decreasing.
   // The actual line returned might not contain 'y', but if not, it is
   // guaranteed to be before any line which does contain 'y'.
   nsLineBox* GetFirstLineContaining(nscoord y);
-  // Set the line cursor to our first line. Only call this if you
-  // guarantee that either the lines' combinedArea.ys and combinedArea.
-  // yMosts are non-decreasing, or the line cursor is cleared before
-  // building the display list of this frame.
-  void SetupLineCursor();
 
-  /**
-   * Helper RAII class for automatically set and clear line cursor for
-   * temporary use. If the frame already has line cursor, this would be
-   * a no-op.
-   */
-  class MOZ_STACK_CLASS AutoLineCursorSetup {
-   public:
-    explicit AutoLineCursorSetup(nsBlockFrame* aFrame)
-        : mFrame(aFrame), mOrigCursor(aFrame->GetLineCursor()) {
-      if (!mOrigCursor) {
-        mFrame->SetupLineCursor();
-      }
-    }
-    ~AutoLineCursorSetup() {
-      if (mOrigCursor) {
-        mFrame->SetProperty(LineCursorProperty(), mOrigCursor);
-      } else {
-        mFrame->ClearLineCursor();
-      }
-    }
+  // Ensure the frame has a display-list line cursor, initializing it to the
+  // first line if it is not already present. (If there's an existing cursor,
+  // it is left untouched.) Only call this if you guarantee that the lines'
+  // combinedArea.ys and combinedArea.yMosts are non-decreasing.
+  void SetupLineCursorForDisplay();
 
-   private:
-    nsBlockFrame* mFrame;
-    nsLineBox* mOrigCursor;
-  };
+  // Ensure the frame has a query line cursor, initializing it to the first
+  // line if it is not already present. (If there's an existing cursor, it is
+  // left untouched.)
+  void SetupLineCursorForQuery();
 
   void ChildIsDirty(nsIFrame* aChild) override;
 
@@ -296,27 +295,21 @@ class nsBlockFrame : public nsContainerFrame {
   /**
    * Compute the final block size of this frame.
    *
-   * @param aReflowInput Data structure passed from parent during reflow.
-   * @param aStatus [in/out] The reflow status for this reflow operation. When
-   *        this function is called, aStatus should represent what our status
-   *        would be as if we were shrinkwrapping our children's block-size.
-   *        This function will then adjust aStatus before returning, if our
-   *        status is different in light of our actual final block-size and
-   *        current page/column's available block-size.
+   * @param aState BlockReflowState passed from parent during reflow.
+   *        Note: aState.mReflowStatus is mostly an "input" parameter. When this
+   *        method is called, it should represent what our status would be as if
+   *        we were shrinkwrapping our children's block-size. This method will
+   *        then adjust it before returning if our status is different in light
+   *        of our actual final block-size and current page/column's available
+   *        block-size.
    * @param aBEndEdgeOfChildren The distance between this frame's block-start
    *        border-edge and the block-end edge of our last child's border-box.
    *        This is effectively our block-start border-padding plus the
    *        block-size of our children, precomputed outside of this function.
-   * @param aBorderPadding The margins representing the border padding for block
-   *        frames. Can be 0.
-   * @param aConsumed The block-size already consumed by our previous-in-flows.
    * @return our final block-size with respect to aReflowInput's writing-mode.
    */
-  nscoord ComputeFinalBSize(const ReflowInput& aReflowInput,
-                            nsReflowStatus& aStatus,
-                            nscoord aBEndEdgeOfChildren,
-                            const mozilla::LogicalMargin& aBorderPadding,
-                            nscoord aConsumed);
+  nscoord ComputeFinalBSize(BlockReflowState& aState,
+                            nscoord aBEndEdgeOfChildren);
 
   void Reflow(nsPresContext* aPresContext, ReflowOutput& aDesiredSize,
               const ReflowInput& aReflowInput,
@@ -328,7 +321,7 @@ class nsBlockFrame : public nsContainerFrame {
    */
   bool DrainSelfOverflowList() override;
 
-  nsresult StealFrame(nsIFrame* aChild) override;
+  void StealFrame(nsIFrame* aChild) override;
 
   void DeleteNextInFlowChild(nsIFrame* aNextInFlow,
                              bool aDeletingEmptyFrames) override;
@@ -348,7 +341,7 @@ class nsBlockFrame : public nsContainerFrame {
    */
   bool CheckForCollapsedBEndMarginFromClearanceLine();
 
-  static nsresult GetCurrentLine(BlockReflowInput* aState,
+  static nsresult GetCurrentLine(BlockReflowState* aState,
                                  nsLineBox** aOutCurrentLine);
 
   /**
@@ -362,23 +355,27 @@ class nsBlockFrame : public nsContainerFrame {
    * Returns whether aFrame is a block frame that will wrap its contents
    * around floats intruding on it from the outside.  (aFrame need not
    * be a block frame, but if it's not, the result will be false.)
+   *
+   * Note: We often use the term "float-avoiding block" to refer to
+   * block-level frames for whom this function returns false.
    */
   static bool BlockCanIntersectFloats(nsIFrame* aFrame);
 
   /**
    * Returns the inline size that needs to be cleared past floats for
-   * blocks that cannot intersect floats.  aState must already have
-   * GetFloatAvailableSpace called on it for the block-dir position that we
-   * care about (which need not be its current mBCoord)
+   * blocks that avoid (i.e. cannot intersect) floats.  aState must already
+   * have GetFloatAvailableSpace called on it for the block-dir position that
+   * we care about (which need not be its current mBCoord)
    */
-  struct ReplacedElementISizeToClear {
+  struct FloatAvoidingISizeToClear {
     // Note that we care about the inline-start margin but can ignore
     // the inline-end margin.
     nscoord marginIStart, borderBoxISize;
   };
-  static ReplacedElementISizeToClear ISizeToClearPastFloats(
-      const BlockReflowInput& aState,
-      const mozilla::LogicalRect& aFloatAvailableSpace, nsIFrame* aFrame);
+  static FloatAvoidingISizeToClear ISizeToClearPastFloats(
+      const BlockReflowState& aState,
+      const mozilla::LogicalRect& aFloatAvailableSpace,
+      nsIFrame* aFloatAvoidingBlock);
 
   /**
    * Creates a contination for aFloat and adds it to the list of overflow
@@ -386,7 +383,7 @@ class nsBlockFrame : public nsContainerFrame {
    * incompleteness. Must only be called while this block frame is in reflow.
    * aFloatStatus must be the float's true, unmodified reflow status.
    */
-  void SplitFloat(BlockReflowInput& aState, nsIFrame* aFloat,
+  void SplitFloat(BlockReflowState& aState, nsIFrame* aFloat,
                   const nsReflowStatus& aFloatStatus);
 
   /**
@@ -426,10 +423,23 @@ class nsBlockFrame : public nsContainerFrame {
       nsPresContext* aPresContext);
 #endif
 
-  NS_DECLARE_FRAME_PROPERTY_WITHOUT_DTOR(LineCursorProperty, nsLineBox)
-  bool HasLineCursor() { return GetStateBits() & NS_BLOCK_HAS_LINE_CURSOR; }
-  nsLineBox* GetLineCursor() {
-    return HasLineCursor() ? GetProperty(LineCursorProperty()) : nullptr;
+  NS_DECLARE_FRAME_PROPERTY_WITHOUT_DTOR(LineCursorPropertyDisplay, nsLineBox)
+  NS_DECLARE_FRAME_PROPERTY_WITHOUT_DTOR(LineCursorPropertyQuery, nsLineBox)
+  // Note that the NS_BLOCK_HAS_LINE_CURSOR flag does not necessarily mean the
+  // cursor is present, as it covers both the "display" and "query" cursors,
+  // but may remain set if they have been separately deleted. In such a case,
+  // the Get* accessors will be slightly more expensive, but will still safely
+  // return null if the cursor is absent.
+  bool MaybeHasLineCursor() {
+    return HasAnyStateBits(NS_BLOCK_HAS_LINE_CURSOR);
+  }
+  nsLineBox* GetLineCursorForDisplay() {
+    return MaybeHasLineCursor() ? GetProperty(LineCursorPropertyDisplay())
+                                : nullptr;
+  }
+  nsLineBox* GetLineCursorForQuery() {
+    return MaybeHasLineCursor() ? GetProperty(LineCursorPropertyQuery())
+                                : nullptr;
   }
 
   nsLineBox* NewLineBox(nsIFrame* aFrame, bool aIsBlock) {
@@ -440,8 +450,11 @@ class nsBlockFrame : public nsContainerFrame {
     return NS_NewLineBox(PresShell(), aFromLine, aFrame, aCount);
   }
   void FreeLineBox(nsLineBox* aLine) {
-    if (aLine == GetLineCursor()) {
-      ClearLineCursor();
+    if (aLine == GetLineCursorForDisplay()) {
+      ClearLineCursorForDisplay();
+    }
+    if (aLine == GetLineCursorForQuery()) {
+      ClearLineCursorForQuery();
     }
     aLine->Destroy(PresShell());
   }
@@ -459,7 +472,7 @@ class nsBlockFrame : public nsContainerFrame {
   /** move the frames contained by aLine by aDeltaBCoord
    * if aLine is a block, its child floats are added to the state manager
    */
-  void SlideLine(BlockReflowInput& aState, nsLineBox* aLine,
+  void SlideLine(BlockReflowState& aState, nsLineBox* aLine,
                  nscoord aDeltaBCoord);
 
   void UpdateLineContainerSize(nsLineBox* aLine,
@@ -469,13 +482,24 @@ class nsBlockFrame : public nsContainerFrame {
   void MoveChildFramesOfLine(nsLineBox* aLine, nscoord aDeltaBCoord);
 
   void ComputeFinalSize(const ReflowInput& aReflowInput,
-                        BlockReflowInput& aState, ReflowOutput& aMetrics,
+                        BlockReflowState& aState, ReflowOutput& aMetrics,
                         nscoord* aBEndEdgeOfChildren);
 
-  void ComputeOverflowAreas(const nsRect& aBounds,
-                            const nsStyleDisplay* aDisplay,
+  /**
+   * Helper method for Reflow(). Computes the overflow areas created by our
+   * children, and includes them into aOverflowAreas.
+   */
+  void ComputeOverflowAreas(mozilla::OverflowAreas& aOverflowAreas,
                             nscoord aBEndEdgeOfChildren,
-                            nsOverflowAreas& aOverflowAreas);
+                            const nsStyleDisplay* aDisplay) const;
+
+  /**
+   * Helper method for ComputeOverflowAreas(). Incorporates aBEndEdgeOfChildren
+   * into the aOverflowAreas.
+   */
+  void ConsiderBlockEndEdgeOfChildren(mozilla::OverflowAreas& aOverflowAreas,
+                                      nscoord aBEndEdgeOfChildren,
+                                      const nsStyleDisplay* aDisplay) const;
 
   /**
    * Add the frames in aFrameList to this block after aPrevSibling.
@@ -535,9 +559,11 @@ class nsBlockFrame : public nsContainerFrame {
   void ReparentFloats(nsIFrame* aFirstFrame, nsBlockFrame* aOldParent,
                       bool aReparentSiblings);
 
-  virtual bool ComputeCustomOverflow(nsOverflowAreas& aOverflowAreas) override;
+  virtual bool ComputeCustomOverflow(
+      mozilla::OverflowAreas& aOverflowAreas) override;
 
-  virtual void UnionChildOverflow(nsOverflowAreas& aOverflowAreas) override;
+  virtual void UnionChildOverflow(
+      mozilla::OverflowAreas& aOverflowAreas) override;
 
   /**
    * Load all of aFrame's floats into the float manager iff aFrame is not a
@@ -560,7 +586,7 @@ class nsBlockFrame : public nsContainerFrame {
     if (!mFloats.IsEmpty()) {
       // If we have pushed floats, then they should be at the beginning of our
       // float list.
-      if (mFloats.FirstChild()->GetStateBits() & NS_FRAME_IS_PUSHED_FLOAT) {
+      if (mFloats.FirstChild()->HasAnyStateBits(NS_FRAME_IS_PUSHED_FLOAT)) {
         return true;
       }
     }
@@ -570,7 +596,7 @@ class nsBlockFrame : public nsContainerFrame {
     // beginning of our floats list.
     for (nsFrameList::Enumerator e(mFloats); !e.AtEnd(); e.Next()) {
       nsIFrame* f = e.get();
-      NS_ASSERTION(!(f->GetStateBits() & NS_FRAME_IS_PUSHED_FLOAT),
+      NS_ASSERTION(!f->HasAnyStateBits(NS_FRAME_IS_PUSHED_FLOAT),
                    "pushed floats must be at the beginning of the float list");
     }
 #endif
@@ -621,7 +647,7 @@ class nsBlockFrame : public nsContainerFrame {
     }
     // For the OverflowOutOfFlowsProperty I think we do enforce that, but it's
     // a mix of out-of-flow frames, so that's why the method name has "Maybe".
-    return GetStateBits() & NS_BLOCK_HAS_OVERFLOW_OUT_OF_FLOWS;
+    return HasAnyStateBits(NS_BLOCK_HAS_OVERFLOW_OUT_OF_FLOWS);
   }
 
   /**
@@ -643,9 +669,8 @@ class nsBlockFrame : public nsContainerFrame {
 
   /** Reflow pushed floats
    */
-  void ReflowPushedFloats(BlockReflowInput& aState,
-                          nsOverflowAreas& aOverflowAreas,
-                          nsReflowStatus& aStatus);
+  void ReflowPushedFloats(BlockReflowState& aState,
+                          mozilla::OverflowAreas& aOverflowAreas);
 
   /** Find any trailing BR clear from the last line of the block (or its PIFs)
    */
@@ -675,10 +700,10 @@ class nsBlockFrame : public nsContainerFrame {
   /** set up the conditions necessary for an resize reflow
    * the primary task is to mark the minimumly sufficient lines dirty.
    */
-  void PrepareResizeReflow(BlockReflowInput& aState);
+  void PrepareResizeReflow(BlockReflowState& aState);
 
   /** reflow all lines that have been marked dirty */
-  void ReflowDirtyLines(BlockReflowInput& aState);
+  void ReflowDirtyLines(BlockReflowState& aState);
 
   /** Mark a given line dirty due to reflow being interrupted on or before it */
   void MarkLineDirtyForInterrupt(nsLineBox* aLine);
@@ -696,13 +721,13 @@ class nsBlockFrame : public nsContainerFrame {
    * @param aKeepReflowGoing [OUT]
    *   indicates whether the caller should continue to reflow more lines
    */
-  void ReflowLine(BlockReflowInput& aState, LineIterator aLine,
+  void ReflowLine(BlockReflowState& aState, LineIterator aLine,
                   bool* aKeepReflowGoing);
 
   // Return false if it needs another reflow because of reduced space
   // between floats that are next to it (but not next to its top), and
   // return true otherwise.
-  bool PlaceLine(BlockReflowInput& aState, nsLineLayout& aLineLayout,
+  bool PlaceLine(BlockReflowState& aState, nsLineLayout& aLineLayout,
                  LineIterator aLine,
                  nsFloatManager::SavedState* aFloatStateBeforeLine,
                  nsFlowAreaRect& aFlowArea,      // in-out
@@ -726,45 +751,45 @@ class nsBlockFrame : public nsContainerFrame {
   void MarkLineDirty(LineIterator aLine, const nsLineList* aLineList);
 
   // XXX where to go
-  bool IsLastLine(BlockReflowInput& aState, LineIterator aLine);
+  bool IsLastLine(BlockReflowState& aState, LineIterator aLine);
 
-  void DeleteLine(BlockReflowInput& aState, nsLineList::iterator aLine,
+  void DeleteLine(BlockReflowState& aState, nsLineList::iterator aLine,
                   nsLineList::iterator aLineEnd);
 
   //----------------------------------------
   // Methods for individual frame reflow
 
-  bool ShouldApplyBStartMargin(BlockReflowInput& aState, nsLineBox* aLine);
+  bool ShouldApplyBStartMargin(BlockReflowState& aState, nsLineBox* aLine);
 
-  void ReflowBlockFrame(BlockReflowInput& aState, LineIterator aLine,
+  void ReflowBlockFrame(BlockReflowState& aState, LineIterator aLine,
                         bool* aKeepGoing);
 
-  void ReflowInlineFrames(BlockReflowInput& aState, LineIterator aLine,
+  void ReflowInlineFrames(BlockReflowState& aState, LineIterator aLine,
                           bool* aKeepLineGoing);
 
   void DoReflowInlineFrames(
-      BlockReflowInput& aState, nsLineLayout& aLineLayout, LineIterator aLine,
+      BlockReflowState& aState, nsLineLayout& aLineLayout, LineIterator aLine,
       nsFlowAreaRect& aFloatAvailableSpace, nscoord& aAvailableSpaceBSize,
       nsFloatManager::SavedState* aFloatStateBeforeLine, bool* aKeepReflowGoing,
       LineReflowStatus* aLineReflowStatus, bool aAllowPullUp);
 
-  void ReflowInlineFrame(BlockReflowInput& aState, nsLineLayout& aLineLayout,
+  void ReflowInlineFrame(BlockReflowState& aState, nsLineLayout& aLineLayout,
                          LineIterator aLine, nsIFrame* aFrame,
                          LineReflowStatus* aLineReflowStatus);
 
-  // Compute the available inline size for a float.
+  // Compute the available size for a float.
   mozilla::LogicalRect AdjustFloatAvailableSpace(
-      BlockReflowInput& aState,
-      const mozilla::LogicalRect& aFloatAvailableSpace, nsIFrame* aFloatFrame);
+      BlockReflowState& aState,
+      const mozilla::LogicalRect& aFloatAvailableSpace);
   // Computes the border-box inline size of the float
-  nscoord ComputeFloatISize(BlockReflowInput& aState,
+  nscoord ComputeFloatISize(BlockReflowState& aState,
                             const mozilla::LogicalRect& aFloatAvailableSpace,
                             nsIFrame* aFloat);
   // An incomplete aReflowStatus indicates the float should be split
   // but only if the available height is constrained.
   // aAdjustedAvailableSpace is the result of calling
   // nsBlockFrame::AdjustFloatAvailableSpace.
-  void ReflowFloat(BlockReflowInput& aState,
+  void ReflowFloat(BlockReflowState& aState,
                    const mozilla::LogicalRect& aAdjustedAvailableSpace,
                    nsIFrame* aFloat, mozilla::LogicalMargin& aFloatMargin,
                    mozilla::LogicalMargin& aFloatOffsets,
@@ -779,12 +804,12 @@ class nsBlockFrame : public nsContainerFrame {
   /**
    * Create a next-in-flow, if necessary, for aFrame. If a new frame is
    * created, place it in aLine if aLine is not null.
-   * @param aState the block reflow input
+   * @param aState the block reflow state
    * @param aLine where to put a new frame
    * @param aFrame the frame
    * @return true if a new frame was created, false if not
    */
-  bool CreateContinuationFor(BlockReflowInput& aState, nsLineBox* aLine,
+  bool CreateContinuationFor(BlockReflowState& aState, nsLineBox* aLine,
                              nsIFrame* aFrame);
 
   /**
@@ -792,10 +817,10 @@ class nsBlockFrame : public nsContainerFrame {
    * page/column.  Set aKeepReflowGoing to false and set
    * flag aState.mReflowStatus as incomplete.
    */
-  void PushTruncatedLine(BlockReflowInput& aState, LineIterator aLine,
+  void PushTruncatedLine(BlockReflowState& aState, LineIterator aLine,
                          bool* aKeepReflowGoing);
 
-  void SplitLine(BlockReflowInput& aState, nsLineLayout& aLineLayout,
+  void SplitLine(BlockReflowState& aState, nsLineLayout& aLineLayout,
                  LineIterator aLine, nsIFrame* aFrame,
                  LineReflowStatus* aLineReflowStatus);
 
@@ -804,7 +829,7 @@ class nsBlockFrame : public nsContainerFrame {
    * one of our next-in-flows lines).
    * @return the pulled frame or nullptr
    */
-  nsIFrame* PullFrame(BlockReflowInput& aState, LineIterator aLine);
+  nsIFrame* PullFrame(BlockReflowState& aState, LineIterator aLine);
 
   /**
    * Try to pull a frame out of a line pointed at by aFromLine.
@@ -826,26 +851,27 @@ class nsBlockFrame : public nsContainerFrame {
    * @param aLineBefore a line in 'mLines' (or LinesBegin() when
    *        pushing the first line)
    */
-  void PushLines(BlockReflowInput& aState, nsLineList::iterator aLineBefore);
+  void PushLines(BlockReflowState& aState, nsLineList::iterator aLineBefore);
 
-  void PropagateFloatDamage(BlockReflowInput& aState, nsLineBox* aLine,
+  void PropagateFloatDamage(BlockReflowState& aState, nsLineBox* aLine,
                             nscoord aDeltaBCoord);
 
-  void CheckFloats(BlockReflowInput& aState);
+  void CheckFloats(BlockReflowState& aState);
 
   //----------------------------------------
   // List handling kludge
 
-  void ReflowOutsideMarker(nsIFrame* aMarkerFrame, BlockReflowInput& aState,
+  void ReflowOutsideMarker(nsIFrame* aMarkerFrame, BlockReflowState& aState,
                            ReflowOutput& aMetrics, nscoord aLineTop);
 
   //----------------------------------------
 
-  virtual nsILineIterator* GetLineIterator() override;
+  bool CanProvideLineIterator() const final { return true; }
+  nsILineIterator* GetLineIterator() final;
 
  public:
   bool HasOverflowLines() const {
-    return 0 != (GetStateBits() & NS_BLOCK_HAS_OVERFLOW_LINES);
+    return HasAnyStateBits(NS_BLOCK_HAS_OVERFLOW_LINES);
   }
   FrameLines* GetOverflowLines() const;
 
@@ -899,7 +925,7 @@ class nsBlockFrame : public nsContainerFrame {
    * @return true if this frame has pushed floats.
    */
   bool HasPushedFloats() const {
-    return 0 != (GetStateBits() & NS_BLOCK_HAS_PUSHED_FLOATS);
+    return HasAnyStateBits(NS_BLOCK_HAS_PUSHED_FLOATS);
   }
 
   // Get the pushed floats list, which is used for *temporary* storage
@@ -927,7 +953,7 @@ class nsBlockFrame : public nsContainerFrame {
   // XXXmats blocks rarely have floats, make it a frame property
   nsFrameList mFloats;
 
-  friend class mozilla::BlockReflowInput;
+  friend class mozilla::BlockReflowState;
   friend class nsBlockInFlowLineIterator;
 
 #ifdef DEBUG
