@@ -25,13 +25,12 @@ class CSSSupportsRule : public css::ConditionRule {
 #endif
 
   RawServoSupportsRule* Raw() const { return mRawRule; }
+  void SetRawAfterClone(RefPtr<RawServoSupportsRule>);
 
   // WebIDL interface
-  uint16_t Type() const override { return CSSRule_Binding::SUPPORTS_RULE; }
+  StyleCssRuleType Type() const final;
   void GetCssText(nsACString& aCssText) const final;
   void GetConditionText(nsACString& aConditionText) final;
-  void SetConditionText(const nsACString& aConditionText,
-                        ErrorResult& aRv) final;
 
   size_t SizeOfIncludingThis(MallocSizeOf) const override;
   JSObject* WrapObject(JSContext* aCx,
