@@ -91,8 +91,6 @@ enum class StyleDisplay : uint16_t {
       StyleDisplayFrom(StyleDisplayOutside::Block, StyleDisplayInside::MozBox),
   MozInlineBox =
       StyleDisplayFrom(StyleDisplayOutside::Inline, StyleDisplayInside::MozBox),
-  MozStack =
-      StyleDisplayFrom(StyleDisplayOutside::XUL, StyleDisplayInside::MozStack),
   MozDeck =
       StyleDisplayFrom(StyleDisplayOutside::XUL, StyleDisplayInside::MozDeck),
   MozPopup =
@@ -386,25 +384,27 @@ enum class StyleGridTrackBreadth : uint8_t {
 #define NS_MATHML_DEFAULT_SCRIPT_MIN_SIZE_PT 8
 
 // See nsStyleFont
-#define NS_MATHML_MATHVARIANT_NONE 0
-#define NS_MATHML_MATHVARIANT_NORMAL 1
-#define NS_MATHML_MATHVARIANT_BOLD 2
-#define NS_MATHML_MATHVARIANT_ITALIC 3
-#define NS_MATHML_MATHVARIANT_BOLD_ITALIC 4
-#define NS_MATHML_MATHVARIANT_SCRIPT 5
-#define NS_MATHML_MATHVARIANT_BOLD_SCRIPT 6
-#define NS_MATHML_MATHVARIANT_FRAKTUR 7
-#define NS_MATHML_MATHVARIANT_DOUBLE_STRUCK 8
-#define NS_MATHML_MATHVARIANT_BOLD_FRAKTUR 9
-#define NS_MATHML_MATHVARIANT_SANS_SERIF 10
-#define NS_MATHML_MATHVARIANT_BOLD_SANS_SERIF 11
-#define NS_MATHML_MATHVARIANT_SANS_SERIF_ITALIC 12
-#define NS_MATHML_MATHVARIANT_SANS_SERIF_BOLD_ITALIC 13
-#define NS_MATHML_MATHVARIANT_MONOSPACE 14
-#define NS_MATHML_MATHVARIANT_INITIAL 15
-#define NS_MATHML_MATHVARIANT_TAILED 16
-#define NS_MATHML_MATHVARIANT_LOOPED 17
-#define NS_MATHML_MATHVARIANT_STRETCHED 18
+enum class StyleMathVariant : uint8_t {
+  None = 0,
+  Normal = 1,
+  Bold = 2,
+  Italic = 3,
+  BoldItalic = 4,
+  Script = 5,
+  BoldScript = 6,
+  Fraktur = 7,
+  DoubleStruck = 8,
+  BoldFraktur = 9,
+  SansSerif = 10,
+  BoldSansSerif = 11,
+  SansSerifItalic = 12,
+  SansSerifBoldItalic = 13,
+  Monospace = 14,
+  Initial = 15,
+  Tailed = 16,
+  Looped = 17,
+  Stretched = 18,
+};
 
 // See nsStyleFont::mMathStyle
 #define NS_STYLE_MATH_STYLE_COMPACT 0
@@ -629,10 +629,11 @@ enum class StyleImeMode : uint8_t {
 enum class StyleWindowShadow : uint8_t {
   None,
   Default,
+
+  // These can't be specified in CSS, they get computed from the "default"
+  // value.
   Menu,
   Tooltip,
-  Sheet,
-  Cliprounded,  // clip border to popup border-radius
 };
 
 // dominant-baseline
@@ -648,14 +649,6 @@ enum class StyleDominantBaseline : uint8_t {
   TextBeforeEdge,
 };
 
-// image-rendering
-enum class StyleImageRendering : uint8_t {
-  Auto,
-  Optimizespeed,
-  Optimizequality,
-  CrispEdges,
-};
-
 // mask-type
 enum class StyleMaskType : uint8_t {
   Luminance,
@@ -710,12 +703,6 @@ enum class StyleTextRendering : uint8_t {
   Geometricprecision,
 };
 
-// color-adjust
-enum class StyleColorAdjust : uint8_t {
-  Economy = 0,
-  Exact = 1,
-};
-
 // color-interpolation and color-interpolation-filters
 enum class StyleColorInterpolation : uint8_t {
   Auto = 0,
@@ -747,6 +734,7 @@ enum class StyleBlend : uint8_t {
   Saturation,
   Color,
   Luminosity,
+  PlusLighter,
 };
 
 // composite
