@@ -21,23 +21,22 @@ const CHROME_ONLY_TOGGLES = [
   "-moz-windows-non-native-menus",
   "-moz-swipe-animation-enabled",
   "-moz-gtk-csd-available",
-  "-moz-gtk-csd-hide-titlebar-by-default",
-  "-moz-gtk-csd-transparent-background",
   "-moz-gtk-csd-minimize-button",
   "-moz-gtk-csd-maximize-button",
   "-moz-gtk-csd-close-button",
   "-moz-gtk-csd-reversed-placement",
-  "-moz-proton",
-  "-moz-proton-places-tooltip",
 ];
 
 // Non-parseable queries can be tested directly in
 // `test_chrome_only_media_queries.html`.
 const CHROME_ONLY_QUERIES = [
-  "(-moz-os-version: windows-win7)",
-  "(-moz-os-version: windows-win8)",
-  "(-moz-os-version: windows-win10)",
-  "(-moz-toolbar-prefers-color-scheme: dark)",
-  "(-moz-toolbar-prefers-color-scheme: light)",
-  "(-moz-toolbar-prefers-color-scheme: system)",
+  "(-moz-platform: linux)",
+  "(-moz-platform: windows)",
+  "(-moz-platform: macos)",
+  "(-moz-platform: android)",
+  "(-moz-platform: windows-win7)",
+  "(-moz-platform: windows-win8)",
+  "(-moz-platform: windows-win10)",
+  "(-moz-content-prefers-color-scheme: dark)",
+  "(-moz-content-prefers-color-scheme: light)",
 ];
