@@ -3,18 +3,18 @@
 
 "use strict";
 
-var { CrashStore, CrashManager } = ChromeUtils.import(
-  "resource://gre/modules/CrashManager.jsm",
-  null
+const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+const { CrashManager } = ChromeUtils.import(
+  "resource://gre/modules/CrashManager.jsm"
 );
-ChromeUtils.import("resource://gre/modules/osfile.jsm", this);
-ChromeUtils.import("resource://gre/modules/Services.jsm", this);
-ChromeUtils.import("resource://gre/modules/TelemetryEnvironment.jsm", this);
-
-ChromeUtils.import("resource://testing-common/CrashManagerTest.jsm", this);
-ChromeUtils.import(
-  "resource://testing-common/TelemetryArchiveTesting.jsm",
-  this
+const { TelemetryArchiveTesting } = ChromeUtils.import(
+  "resource://testing-common/TelemetryArchiveTesting.jsm"
+);
+const { configureLogging, getManager, sleep } = ChromeUtils.import(
+  "resource://testing-common/CrashManagerTest.jsm"
+);
+const { TelemetryEnvironment } = ChromeUtils.import(
+  "resource://gre/modules/TelemetryEnvironment.jsm"
 );
 
 const DUMMY_DATE = new Date(Date.now() - 10 * 24 * 60 * 60 * 1000);
@@ -54,6 +54,36 @@ add_task(async function test_get_manager() {
   await m.createDummyDump(false);
 });
 
+add_task(async function test_valid_process() {
+  let m = await getManager();
+  Assert.ok(m, "CrashManager obtained.");
+
+  Assert.ok(!m.isValidProcessType(42));
+  Assert.ok(!m.isValidProcessType(null));
+  Assert.ok(!m.isValidProcessType("default"));
+
+  Assert.ok(m.isValidProcessType("main"));
+});
+
+add_task(async function test_process_ping() {
+  let m = await getManager();
+  Assert.ok(m, "CrashManager obtained.");
+
+  Assert.ok(!m.isPingAllowed(42));
+  Assert.ok(!m.isPingAllowed(null));
+  Assert.ok(!m.isPingAllowed("default"));
+
+  Assert.ok(!m.isPingAllowed("main"));
+  Assert.ok(!m.isPingAllowed("ipdlunittest"));
+  Assert.ok(!m.isPingAllowed("gmplugin"));
+  Assert.ok(!m.isPingAllowed("remotesandboxbroker"));
+  Assert.ok(!m.isPingAllowed("forkserver"));
+
+  Assert.ok(m.isPingAllowed("content"));
+  Assert.ok(m.isPingAllowed("gpu"));
+  Assert.ok(m.isPingAllowed("utility"));
+});
+
 // Unsubmitted dump files on disk are detected properly.
 add_task(async function test_pending_dumps() {
   let m = await getManager();
@@ -119,12 +149,21 @@ add_task(async function test_store_expires() {
   Assert.ok(!m._store, "Store has gone away.");
 });
 
+// Ensure errors are handled when the events dir is missing.
+add_task(async function test_empty_events_dir() {
+  let m = await getManager();
+  await m.deleteEventsDirs();
+
+  let paths = await m._getUnprocessedEventsFiles();
+  Assert.equal(paths.length, 0);
+});
+
 // Ensure discovery of unprocessed events files works.
 add_task(async function test_unprocessed_events_files() {
   let m = await getManager();
-  await m.createEventsFile("1", "test.1", new Date(), "foo", 0);
-  await m.createEventsFile("2", "test.1", new Date(), "bar", 0);
-  await m.createEventsFile("1", "test.1", new Date(), "baz", 1);
+  await m.createEventsFile("1", "test.1", new Date(), "foo", "{}", 0);
+  await m.createEventsFile("2", "test.1", new Date(), "bar", "{}", 0);
+  await m.createEventsFile("1", "test.1", new Date(), "baz", "{}", 1);
 
   let paths = await m._getUnprocessedEventsFiles();
   Assert.equal(paths.length, 3);
@@ -159,7 +198,7 @@ add_task(async function test_malformed_files_deleted() {
 add_task(async function test_aggregate_ignore_unknown_events() {
   let m = await getManager();
 
-  await m.createEventsFile("1", "crash.main.2", DUMMY_DATE, "id1");
+  await m.createEventsFile("1", "crash.main.3", DUMMY_DATE, "id1", "{}");
   await m.createEventsFile("2", "foobar.1", new Date(), "dummy");
 
   let count = await m.aggregateEventsFiles();
@@ -176,8 +215,13 @@ add_task(async function test_prune_old() {
   let m = await getManager();
   let oldDate = new Date(Date.now() - 86400000);
   let newDate = new Date(Date.now() - 10000);
-  await m.createEventsFile("1", "crash.main.2", oldDate, "id1");
-  await m.addCrash(m.PROCESS_TYPE_PLUGIN, m.CRASH_TYPE_CRASH, "id2", newDate);
+  await m.createEventsFile("1", "crash.main.3", oldDate, "id1", "{}");
+  await m.addCrash(
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_CONTENT],
+    m.CRASH_TYPE_CRASH,
+    "id2",
+    newDate
+  );
 
   await m.aggregateEventsFiles();
 
@@ -201,12 +245,12 @@ add_task(async function test_prune_old() {
 
 add_task(async function test_schedule_maintenance() {
   let m = await getManager();
-  await m.createEventsFile("1", "crash.main.2", DUMMY_DATE, "id1");
+  await m.createEventsFile("1", "crash.main.3", DUMMY_DATE, "id1", "{}");
 
   let oldDate = new Date(
     Date.now() - m.PURGE_OLDER_THAN_DAYS * 2 * 24 * 60 * 60 * 1000
   );
-  await m.createEventsFile("2", "crash.main.2", oldDate, "id2");
+  await m.createEventsFile("2", "crash.main.3", oldDate, "id2", "{}");
 
   await m.scheduleMaintenance(25);
   let crashes = await m.getCrashes();
@@ -215,12 +259,11 @@ add_task(async function test_schedule_maintenance() {
 });
 
 const crashId = "3cb67eba-0dc7-6f78-6a569a0e-172287ec";
-const crashPingUuid = "103dbdf2-339b-4b9c-a7cc-5f9506ea9d08";
 const productName = "Firefox";
 const productId = "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}";
 const sha256Hash =
   "f8410c3ac4496cfa9191a1240f0e365101aef40c7bf34fc5bcb8ec511832ed79";
-const stackTraces = '{"status":"OK"}';
+const stackTraces = { status: "OK" };
 
 add_task(async function test_main_crash_event_file() {
   let ac = new TelemetryArchiveTesting.Checker();
@@ -233,30 +276,23 @@ add_task(async function test_main_crash_event_file() {
   theEnvironment.testValue = 'MyValue"';
 
   let m = await getManager();
-  const fileContent =
-    crashId +
-    "\n" +
-    "ProductName=" +
-    productName +
-    "\n" +
-    "ProductID=" +
-    productId +
-    "\n" +
-    "TelemetryEnvironment=" +
-    JSON.stringify(theEnvironment) +
-    "\n" +
-    "TelemetrySessionId=" +
-    sessionId +
-    "\n" +
-    "MinidumpSha256Hash=" +
-    sha256Hash +
-    "\n" +
-    "StackTraces=" +
-    stackTraces +
-    "\n" +
-    "ThisShouldNot=end-up-in-the-ping\n";
+  const metadata = JSON.stringify({
+    ProductName: productName,
+    ProductID: productId,
+    TelemetryEnvironment: JSON.stringify(theEnvironment),
+    TelemetrySessionId: sessionId,
+    MinidumpSha256Hash: sha256Hash,
+    StackTraces: stackTraces,
+    ThisShouldNot: "end-up-in-the-ping",
+  });
 
-  await m.createEventsFile(crashId, "crash.main.2", DUMMY_DATE, fileContent);
+  await m.createEventsFile(
+    crashId,
+    "crash.main.3",
+    DUMMY_DATE,
+    crashId,
+    metadata
+  );
   let count = await m.aggregateEventsFiles();
   Assert.equal(count, 1);
 
@@ -290,7 +326,7 @@ add_task(async function test_main_crash_event_file() {
   Assert.equal(
     found.payload.metadata.ThisShouldNot,
     undefined,
-    "Non-whitelisted fields should be filtered out"
+    "Non-allowlisted fields should be filtered out"
   );
 
   count = await m.aggregateEventsFiles();
@@ -300,18 +336,19 @@ add_task(async function test_main_crash_event_file() {
 add_task(async function test_main_crash_event_file_noenv() {
   let ac = new TelemetryArchiveTesting.Checker();
   await ac.promiseInit();
-  const fileContent =
-    crashId +
-    "\n" +
-    "ProductName=" +
-    productName +
-    "\n" +
-    "ProductID=" +
-    productId +
-    "\n";
+  const metadata = JSON.stringify({
+    ProductName: productName,
+    ProductID: productId,
+  });
 
   let m = await getManager();
-  await m.createEventsFile(crashId, "crash.main.2", DUMMY_DATE, fileContent);
+  await m.createEventsFile(
+    crashId,
+    "crash.main.3",
+    DUMMY_DATE,
+    crashId,
+    metadata
+  );
   let count = await m.aggregateEventsFiles();
   Assert.equal(count, 1);
 
@@ -339,12 +376,13 @@ add_task(async function test_main_crash_event_file_noenv() {
 
 add_task(async function test_crash_submission_event_file() {
   let m = await getManager();
-  await m.createEventsFile("1", "crash.main.2", DUMMY_DATE, "crash1");
+  await m.createEventsFile("1", "crash.main.3", DUMMY_DATE, "crash1", "{}");
   await m.createEventsFile(
     "1-submission",
     "crash.submission.1",
     DUMMY_DATE_2,
-    "crash1\nfalse\n"
+    "crash1",
+    "false\n"
   );
 
   // The line below has been intentionally commented out to make sure that
@@ -354,7 +392,8 @@ add_task(async function test_crash_submission_event_file() {
     "2-submission",
     "crash.submission.1",
     DUMMY_DATE_2,
-    "crash2\ntrue\nbp-2"
+    "crash2",
+    "true\nbp-2"
   );
   let count = await m.aggregateEventsFiles();
   Assert.equal(count, 3);
@@ -402,11 +441,17 @@ add_task(async function test_high_water_mark() {
   let store = await m._getStore();
 
   for (let i = 0; i < store.HIGH_WATER_DAILY_THRESHOLD + 1; i++) {
-    await m.createEventsFile("m" + i, "crash.main.2", DUMMY_DATE, "m" + i);
+    await m.createEventsFile(
+      "m" + i,
+      "crash.main.3",
+      DUMMY_DATE,
+      "m" + i,
+      "{}"
+    );
   }
 
   let count = await m.aggregateEventsFiles();
-  Assert.equal(count, CrashStore.prototype.HIGH_WATER_DAILY_THRESHOLD + 1);
+  Assert.equal(count, store.HIGH_WATER_DAILY_THRESHOLD + 1);
 
   // Need to fetch again in case the first one was garbage collected.
   store = await m._getStore();
@@ -421,176 +466,246 @@ add_task(async function test_addCrash() {
   Assert.equal(crashes.length, 0);
 
   await m.addCrash(
-    m.PROCESS_TYPE_MAIN,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT],
     m.CRASH_TYPE_CRASH,
     "main-crash",
     DUMMY_DATE
   );
   await m.addCrash(
-    m.PROCESS_TYPE_MAIN,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT],
     m.CRASH_TYPE_HANG,
     "main-hang",
     DUMMY_DATE
   );
   await m.addCrash(
-    m.PROCESS_TYPE_CONTENT,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_CONTENT],
     m.CRASH_TYPE_CRASH,
     "content-crash",
     DUMMY_DATE
   );
   await m.addCrash(
-    m.PROCESS_TYPE_CONTENT,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_CONTENT],
     m.CRASH_TYPE_HANG,
     "content-hang",
     DUMMY_DATE
   );
   await m.addCrash(
-    m.PROCESS_TYPE_PLUGIN,
-    m.CRASH_TYPE_CRASH,
-    "plugin-crash",
-    DUMMY_DATE
-  );
-  await m.addCrash(
-    m.PROCESS_TYPE_PLUGIN,
-    m.CRASH_TYPE_HANG,
-    "plugin-hang",
-    DUMMY_DATE
-  );
-  await m.addCrash(
-    m.PROCESS_TYPE_GMPLUGIN,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_GMPLUGIN],
     m.CRASH_TYPE_CRASH,
     "gmplugin-crash",
     DUMMY_DATE
   );
   await m.addCrash(
-    m.PROCESS_TYPE_GPU,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_GPU],
     m.CRASH_TYPE_CRASH,
     "gpu-crash",
     DUMMY_DATE
   );
   await m.addCrash(
-    m.PROCESS_TYPE_VR,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_VR],
     m.CRASH_TYPE_CRASH,
     "vr-crash",
     DUMMY_DATE
   );
   await m.addCrash(
-    m.PROCESS_TYPE_RDD,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_RDD],
     m.CRASH_TYPE_CRASH,
     "rdd-crash",
     DUMMY_DATE
   );
   await m.addCrash(
-    m.PROCESS_TYPE_SOCKET,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_SOCKET],
     m.CRASH_TYPE_CRASH,
     "socket-crash",
     DUMMY_DATE
   );
 
   await m.addCrash(
-    m.PROCESS_TYPE_MAIN,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT],
     m.CRASH_TYPE_CRASH,
     "changing-item",
     DUMMY_DATE
   );
   await m.addCrash(
-    m.PROCESS_TYPE_CONTENT,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_CONTENT],
     m.CRASH_TYPE_HANG,
     "changing-item",
     DUMMY_DATE_2
   );
 
   crashes = await m.getCrashes();
-  Assert.equal(crashes.length, 12);
+  Assert.equal(crashes.length, 10);
 
   let map = new Map(crashes.map(crash => [crash.id, crash]));
 
   let crash = map.get("main-crash");
   Assert.ok(!!crash);
   Assert.equal(crash.crashDate, DUMMY_DATE);
-  Assert.equal(crash.type, m.PROCESS_TYPE_MAIN + "-" + m.CRASH_TYPE_CRASH);
-  Assert.ok(crash.isOfType(m.PROCESS_TYPE_MAIN, m.CRASH_TYPE_CRASH));
+  Assert.equal(
+    crash.type,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT] +
+      "-" +
+      m.CRASH_TYPE_CRASH
+  );
+  Assert.ok(
+    crash.isOfType(
+      m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT],
+      m.CRASH_TYPE_CRASH
+    )
+  );
 
   crash = map.get("main-hang");
   Assert.ok(!!crash);
   Assert.equal(crash.crashDate, DUMMY_DATE);
-  Assert.equal(crash.type, m.PROCESS_TYPE_MAIN + "-" + m.CRASH_TYPE_HANG);
-  Assert.ok(crash.isOfType(m.PROCESS_TYPE_MAIN, m.CRASH_TYPE_HANG));
+  Assert.equal(
+    crash.type,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT] +
+      "-" +
+      m.CRASH_TYPE_HANG
+  );
+  Assert.ok(
+    crash.isOfType(
+      m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT],
+      m.CRASH_TYPE_HANG
+    )
+  );
 
   crash = map.get("content-crash");
   Assert.ok(!!crash);
   Assert.equal(crash.crashDate, DUMMY_DATE);
-  Assert.equal(crash.type, m.PROCESS_TYPE_CONTENT + "-" + m.CRASH_TYPE_CRASH);
-  Assert.ok(crash.isOfType(m.PROCESS_TYPE_CONTENT, m.CRASH_TYPE_CRASH));
+  Assert.equal(
+    crash.type,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_CONTENT] +
+      "-" +
+      m.CRASH_TYPE_CRASH
+  );
+  Assert.ok(
+    crash.isOfType(
+      m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_CONTENT],
+      m.CRASH_TYPE_CRASH
+    )
+  );
 
   crash = map.get("content-hang");
   Assert.ok(!!crash);
   Assert.equal(crash.crashDate, DUMMY_DATE);
-  Assert.equal(crash.type, m.PROCESS_TYPE_CONTENT + "-" + m.CRASH_TYPE_HANG);
-  Assert.ok(crash.isOfType(m.PROCESS_TYPE_CONTENT, m.CRASH_TYPE_HANG));
-
-  crash = map.get("plugin-crash");
-  Assert.ok(!!crash);
-  Assert.equal(crash.crashDate, DUMMY_DATE);
-  Assert.equal(crash.type, m.PROCESS_TYPE_PLUGIN + "-" + m.CRASH_TYPE_CRASH);
-  Assert.ok(crash.isOfType(m.PROCESS_TYPE_PLUGIN, m.CRASH_TYPE_CRASH));
-
-  crash = map.get("plugin-hang");
-  Assert.ok(!!crash);
-  Assert.equal(crash.crashDate, DUMMY_DATE);
-  Assert.equal(crash.type, m.PROCESS_TYPE_PLUGIN + "-" + m.CRASH_TYPE_HANG);
-  Assert.ok(crash.isOfType(m.PROCESS_TYPE_PLUGIN, m.CRASH_TYPE_HANG));
+  Assert.equal(
+    crash.type,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_CONTENT] +
+      "-" +
+      m.CRASH_TYPE_HANG
+  );
+  Assert.ok(
+    crash.isOfType(
+      m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_CONTENT],
+      m.CRASH_TYPE_HANG
+    )
+  );
 
   crash = map.get("gmplugin-crash");
   Assert.ok(!!crash);
   Assert.equal(crash.crashDate, DUMMY_DATE);
-  Assert.equal(crash.type, m.PROCESS_TYPE_GMPLUGIN + "-" + m.CRASH_TYPE_CRASH);
-  Assert.ok(crash.isOfType(m.PROCESS_TYPE_GMPLUGIN, m.CRASH_TYPE_CRASH));
+  Assert.equal(
+    crash.type,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_GMPLUGIN] +
+      "-" +
+      m.CRASH_TYPE_CRASH
+  );
+  Assert.ok(
+    crash.isOfType(
+      m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_GMPLUGIN],
+      m.CRASH_TYPE_CRASH
+    )
+  );
 
   crash = map.get("gpu-crash");
   Assert.ok(!!crash);
   Assert.equal(crash.crashDate, DUMMY_DATE);
-  Assert.equal(crash.type, m.PROCESS_TYPE_GPU + "-" + m.CRASH_TYPE_CRASH);
-  Assert.ok(crash.isOfType(m.PROCESS_TYPE_GPU, m.CRASH_TYPE_CRASH));
+  Assert.equal(
+    crash.type,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_GPU] + "-" + m.CRASH_TYPE_CRASH
+  );
+  Assert.ok(
+    crash.isOfType(
+      m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_GPU],
+      m.CRASH_TYPE_CRASH
+    )
+  );
 
   crash = map.get("vr-crash");
   Assert.ok(!!crash);
   Assert.equal(crash.crashDate, DUMMY_DATE);
-  Assert.equal(crash.type, m.PROCESS_TYPE_VR + "-" + m.CRASH_TYPE_CRASH);
-  Assert.ok(crash.isOfType(m.PROCESS_TYPE_VR, m.CRASH_TYPE_CRASH));
+  Assert.equal(
+    crash.type,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_VR] + "-" + m.CRASH_TYPE_CRASH
+  );
+  Assert.ok(
+    crash.isOfType(
+      m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_VR],
+      m.CRASH_TYPE_CRASH
+    )
+  );
 
   crash = map.get("rdd-crash");
   Assert.ok(!!crash);
   Assert.equal(crash.crashDate, DUMMY_DATE);
-  Assert.equal(crash.type, m.PROCESS_TYPE_RDD + "-" + m.CRASH_TYPE_CRASH);
-  Assert.ok(crash.isOfType(m.PROCESS_TYPE_RDD, m.CRASH_TYPE_CRASH));
+  Assert.equal(
+    crash.type,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_RDD] + "-" + m.CRASH_TYPE_CRASH
+  );
+  Assert.ok(
+    crash.isOfType(
+      m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_RDD],
+      m.CRASH_TYPE_CRASH
+    )
+  );
 
   crash = map.get("socket-crash");
   Assert.ok(!!crash);
   Assert.equal(crash.crashDate, DUMMY_DATE);
-  Assert.equal(crash.type, m.PROCESS_TYPE_SOCKET + "-" + m.CRASH_TYPE_CRASH);
-  Assert.ok(crash.isOfType(m.PROCESS_TYPE_SOCKET, m.CRASH_TYPE_CRASH));
+  Assert.equal(
+    crash.type,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_SOCKET] +
+      "-" +
+      m.CRASH_TYPE_CRASH
+  );
+  Assert.ok(
+    crash.isOfType(
+      m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_SOCKET],
+      m.CRASH_TYPE_CRASH
+    )
+  );
 
   crash = map.get("changing-item");
   Assert.ok(!!crash);
   Assert.equal(crash.crashDate, DUMMY_DATE_2);
-  Assert.equal(crash.type, m.PROCESS_TYPE_CONTENT + "-" + m.CRASH_TYPE_HANG);
-  Assert.ok(crash.isOfType(m.PROCESS_TYPE_CONTENT, m.CRASH_TYPE_HANG));
+  Assert.equal(
+    crash.type,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_CONTENT] +
+      "-" +
+      m.CRASH_TYPE_HANG
+  );
+  Assert.ok(
+    crash.isOfType(
+      m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_CONTENT],
+      m.CRASH_TYPE_HANG
+    )
+  );
 });
 
 add_task(async function test_child_process_crash_ping() {
   let m = await getManager();
   const EXPECTED_PROCESSES = [
-    m.PROCESS_TYPE_CONTENT,
-    m.PROCESS_TYPE_GPU,
-    m.PROCESS_TYPE_VR,
-    m.PROCESS_TYPE_RDD,
-    m.PROCESS_TYPE_SOCKET,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_CONTENT],
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_GPU],
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_VR],
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_RDD],
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_SOCKET],
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_UTILITY],
   ];
 
   const UNEXPECTED_PROCESSES = [
-    m.PROCESS_TYPE_PLUGIN,
-    m.PROCESS_TYPE_GMPLUGIN,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_GMPLUGIN],
     null,
     12, // non-string process type
   ];
@@ -601,7 +716,10 @@ add_task(async function test_child_process_crash_ping() {
   // Add a child-process crash for each allowed process type.
   for (let p of EXPECTED_PROCESSES) {
     // Generate a ping.
-    const remoteType = p === m.PROCESS_TYPE_CONTENT ? "web" : undefined;
+    const remoteType =
+      p === m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_CONTENT]
+        ? "web"
+        : undefined;
     let id = await m.createDummyDump();
     await m.addCrash(p, m.CRASH_TYPE_CRASH, id, DUMMY_DATE, {
       RemoteType: remoteType,
@@ -631,17 +749,17 @@ add_task(async function test_child_process_crash_ping() {
     Assert.equal(
       found.payload.metadata.ThisShouldNot,
       undefined,
-      "Non-whitelisted fields should be filtered out"
+      "Non-allowlisted fields should be filtered out"
     );
     Assert.equal(
       found.payload.metadata.RemoteType,
       remoteType,
-      "RemoteType should be whitelisted for content crashes"
+      "RemoteType should be allowlisted for content crashes"
     );
     Assert.equal(
       found.payload.metadata.ipc_channel_error,
       "ShutDownKill",
-      "ipc_channel_error should be whitelisted for content crashes"
+      "ipc_channel_error should be allowlisted for content crashes"
     );
   }
 
@@ -682,7 +800,7 @@ add_task(async function test_addSubmissionAttemptAndResult() {
   Assert.equal(crashes.length, 0);
 
   await m.addCrash(
-    m.PROCESS_TYPE_MAIN,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT],
     m.CRASH_TYPE_CRASH,
     "main-crash",
     DUMMY_DATE
@@ -729,7 +847,7 @@ add_task(async function test_addSubmissionAttemptEarlyCall() {
     });
 
   await m.addCrash(
-    m.PROCESS_TYPE_MAIN,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT],
     m.CRASH_TYPE_CRASH,
     "main-crash",
     DUMMY_DATE
@@ -753,7 +871,7 @@ add_task(async function test_setCrashClassifications() {
   let m = await getManager();
 
   await m.addCrash(
-    m.PROCESS_TYPE_MAIN,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT],
     m.CRASH_TYPE_CRASH,
     "main-crash",
     DUMMY_DATE
@@ -767,7 +885,7 @@ add_task(async function test_setRemoteCrashID() {
   let m = await getManager();
 
   await m.addCrash(
-    m.PROCESS_TYPE_MAIN,
+    m.processTypes[Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT],
     m.CRASH_TYPE_CRASH,
     "main-crash",
     DUMMY_DATE
@@ -776,6 +894,23 @@ add_task(async function test_setRemoteCrashID() {
   Assert.equal((await m.getCrashes())[0].remoteID, "bp-1");
 });
 
+add_task(async function test_addCrashWrong() {
+  let m = await getManager();
+
+  let crashes = await m.getCrashes();
+  Assert.equal(crashes.length, 0);
+
+  await m.addCrash(
+    m.processTypes[-1], // passing a wrong type to force 'undefined', it should
+    m.CRASH_TYPE_CRASH, // fail in the end and not record it
+    "wrong-content-crash",
+    DUMMY_DATE
+  );
+
+  crashes = await m.getCrashes();
+  Assert.equal(crashes.length, 0);
+});
+
 add_task(async function test_telemetryHistogram() {
   let Telemetry = Services.telemetry;
   let h = Telemetry.getKeyedHistogramById("PROCESS_CRASH_SUBMIT_ATTEMPT");
@@ -786,11 +921,16 @@ add_task(async function test_telemetryHistogram() {
   let processTypes = [];
   let crashTypes = [];
 
-  // Gather all process and crash types
+  // Gather all process types
+  for (let field in m.processTypes) {
+    if (m.isPingAllowed(m.processTypes[field])) {
+      processTypes.push(m.processTypes[field]);
+    }
+  }
+
+  // Gather all crash types
   for (let field in m) {
-    if (field.startsWith("PROCESS_TYPE_")) {
-      processTypes.push(m[field]);
-    } else if (field.startsWith("CRASH_TYPE_")) {
+    if (field.startsWith("CRASH_TYPE_")) {
       crashTypes.push(m[field]);
     }
   }
@@ -808,6 +948,10 @@ add_task(async function test_telemetryHistogram() {
     }
   }
 
+  // Ensure that we have generated some crash, otherwise it could indicate
+  // something silently regressing
+  Assert.greater(keysCount, 2);
+
   // Check that we have the expected keys.
   let snap = h.snapshot();
   Assert.equal(
