@@ -7,7 +7,6 @@
 const bmsvc = PlacesUtils.bookmarks;
 const obsvc = PlacesUtils.observers;
 const tagssvc = PlacesUtils.tagging;
-const annosvc = PlacesUtils.annotations;
 const PT = PlacesTransactions;
 const menuGuid = PlacesUtils.bookmarks.menuGuid;
 
@@ -28,57 +27,72 @@ var observer = {
     this.itemsRemoved = new Map();
     this.itemsChanged = new Map();
     this.itemsMoved = new Map();
-    this.beginUpdateBatch = false;
-    this.endUpdateBatch = false;
+    this.itemsTitleChanged = new Map();
+    this.itemsUrlChanged = new Map();
   },
 
   handlePlacesEvents(events) {
     for (let event of events) {
-      // Ignore tag items.
-      if (event.isTagging) {
-        this.tagRelatedGuids.add(event.guid);
-        return;
+      switch (event.type) {
+        case "bookmark-added":
+          // Ignore tag items.
+          if (event.isTagging) {
+            this.tagRelatedGuids.add(event.guid);
+            return;
+          }
+
+          this.itemsAdded.set(event.guid, {
+            itemId: event.id,
+            parentGuid: event.parentGuid,
+            index: event.index,
+            itemType: event.itemType,
+            title: event.title,
+            url: event.url,
+          });
+          break;
+        case "bookmark-removed":
+          if (this.tagRelatedGuids.has(event.guid)) {
+            return;
+          }
+
+          this.itemsRemoved.set(event.guid, {
+            parentGuid: event.parentGuid,
+            index: event.index,
+            itemType: event.itemType,
+          });
+          break;
+        case "bookmark-moved":
+          this.itemsMoved.set(event.guid, {
+            oldParentGuid: event.oldParentGuid,
+            oldIndex: event.oldIndex,
+            newParentGuid: event.parentGuid,
+            newIndex: event.index,
+            itemType: event.itemType,
+          });
+          break;
+        case "bookmark-title-changed":
+          if (this.tagRelatedGuids.has(event.guid)) {
+            return;
+          }
+
+          this.itemsTitleChanged.set(event.guid, {
+            title: event.title,
+            parentGuid: event.parentGuid,
+          });
+          break;
+        case "bookmark-url-changed":
+          if (this.tagRelatedGuids.has(event.guid)) {
+            return;
+          }
+
+          this.itemsUrlChanged.set(event.guid, {
+            url: event.url,
+          });
+          break;
       }
-
-      this.itemsAdded.set(event.guid, {
-        itemId: event.id,
-        parentGuid: event.parentGuid,
-        index: event.index,
-        itemType: event.itemType,
-        title: event.title,
-        url: event.url,
-      });
     }
   },
 
-  onBeginUpdateBatch() {
-    this.beginUpdateBatch = true;
-  },
-
-  onEndUpdateBatch() {
-    this.endUpdateBatch = true;
-  },
-
-  onItemRemoved(
-    aItemId,
-    aParentId,
-    aIndex,
-    aItemType,
-    aURI,
-    aGuid,
-    aParentGuid
-  ) {
-    if (this.tagRelatedGuids.has(aGuid)) {
-      return;
-    }
-
-    this.itemsRemoved.set(aGuid, {
-      parentGuid: aParentGuid,
-      index: aIndex,
-      itemType: aItemType,
-    });
-  },
-
   onItemChanged(
     aItemId,
     aProperty,
@@ -99,45 +113,14 @@ var observer = {
       changesForGuid = new Map();
       this.itemsChanged.set(aGuid, changesForGuid);
     }
-
-    let newValue = aNewValue;
-    if (aIsAnnoProperty) {
-      if (annosvc.itemHasAnnotation(aItemId, aProperty)) {
-        newValue = annosvc.getItemAnnotation(aItemId, aProperty);
-      } else {
-        newValue = null;
-      }
-    }
     let change = {
       isAnnoProperty: aIsAnnoProperty,
-      newValue,
+      newValue: aNewValue,
       lastModified: aLastModified,
       itemType: aItemType,
     };
     changesForGuid.set(aProperty, change);
   },
-
-  onItemVisited: () => {},
-
-  onItemMoved(
-    aItemId,
-    aOldParent,
-    aOldIndex,
-    aNewParent,
-    aNewIndex,
-    aItemType,
-    aGuid,
-    aOldParentGuid,
-    aNewParentGuid
-  ) {
-    this.itemsMoved.set(aGuid, {
-      oldParentGuid: aOldParentGuid,
-      oldIndex: aOldIndex,
-      newParentGuid: aNewParentGuid,
-      newIndex: aNewIndex,
-      itemType: aItemType,
-    });
-  },
 };
 observer.reset();
 
@@ -147,10 +130,28 @@ var bmStartIndex = 0;
 function run_test() {
   bmsvc.addObserver(observer);
   observer.handlePlacesEvents = observer.handlePlacesEvents.bind(observer);
-  obsvc.addListener(["bookmark-added"], observer.handlePlacesEvents);
+  obsvc.addListener(
+    [
+      "bookmark-added",
+      "bookmark-removed",
+      "bookmark-moved",
+      "bookmark-title-changed",
+      "bookmark-url-changed",
+    ],
+    observer.handlePlacesEvents
+  );
   registerCleanupFunction(function() {
     bmsvc.removeObserver(observer);
-    obsvc.removeListener(["bookmark-added"], observer.handlePlacesEvents);
+    obsvc.removeListener(
+      [
+        "bookmark-added",
+        "bookmark-removed",
+        "bookmark-moved",
+        "bookmark-title-changed",
+        "bookmark-url-changed",
+      ],
+      observer.handlePlacesEvents
+    );
   });
 
   run_next_test();
@@ -287,11 +288,7 @@ function ensureItemsChanged(...items) {
     let changes = observer.itemsChanged.get(item.guid);
     Assert.ok(changes.has(item.property));
     let info = changes.get(item.property);
-    if (!("isAnnoProperty" in item)) {
-      Assert.ok(!info.isAnnoProperty);
-    } else {
-      Assert.equal(info.isAnnoProperty, Boolean(item.isAnnoProperty));
-    }
+    Assert.ok(!info.isAnnoProperty);
     Assert.equal(info.newValue, item.newValue);
     if ("url" in item) {
       Assert.ok(item.url.equals(info.url));
@@ -299,17 +296,6 @@ function ensureItemsChanged(...items) {
   }
 }
 
-function ensureAnnotationsSet(aGuid, aAnnos) {
-  Assert.ok(observer.itemsChanged.has(aGuid));
-  let changes = observer.itemsChanged.get(aGuid);
-  for (let anno of aAnnos) {
-    Assert.ok(changes.has(anno.name));
-    let changeInfo = changes.get(anno.name);
-    Assert.ok(changeInfo.isAnnoProperty);
-    Assert.equal(changeInfo.newValue, anno.value);
-  }
-}
-
 function ensureItemsMoved(...items) {
   Assert.equal(
     observer.itemsMoved.size,
@@ -345,13 +331,41 @@ function ensureItemsMoved(...items) {
   }
 }
 
-function ensureTimestampsUpdated(aGuid, aCheckDateAdded = false) {
-  Assert.ok(observer.itemsChanged.has(aGuid));
-  let changes = observer.itemsChanged.get(aGuid);
-  if (aCheckDateAdded) {
-    Assert.ok(changes.has("dateAdded"));
+function ensureItemsTitleChanged(...items) {
+  Assert.equal(
+    observer.itemsTitleChanged.size,
+    items.length,
+    "Should have received the correct number of bookmark-title-changed notifications"
+  );
+  for (const item of items) {
+    Assert.ok(
+      observer.itemsTitleChanged.has(item.guid),
+      `Observer should have a title changed for ${item.guid}`
+    );
+    const info = observer.itemsTitleChanged.get(item.guid);
+    Assert.equal(info.title, item.title, "Should have the correct title");
+    Assert.equal(
+      info.parentGuid,
+      item.parentGuid,
+      "Should have the correct parent guid"
+    );
+  }
+}
+
+function ensureItemsUrlChanged(...items) {
+  Assert.equal(
+    observer.itemsUrlChanged.size,
+    items.length,
+    "Should have received the correct number of bookmark-url-changed notifications"
+  );
+  for (const item of items) {
+    Assert.ok(
+      observer.itemsUrlChanged.has(item.guid),
+      `Observer should have a title changed for ${item.guid}`
+    );
+    const info = observer.itemsUrlChanged.get(item.guid);
+    Assert.equal(info.url, item.url, "Should have the correct url");
   }
-  Assert.ok(changes.has("lastModified"));
 }
 
 function ensureTagsForURI(aURI, aTags) {
@@ -1231,7 +1245,7 @@ add_task(async function test_add_and_remove_bookmarks_with_additional_info() {
   await ensureItemsRemoved(b2_info);
   ensureTags([TAG_1]);
 
-  // Check if Remove correctly restores tags and annotations.
+  // Check if Remove correctly restores tags.
   observer.reset();
   await PT.redo();
   ensureTags([TAG_1, TAG_2]);
@@ -1266,7 +1280,34 @@ add_task(async function test_add_and_remove_bookmarks_with_additional_info() {
 
   observer.reset();
   await PT.redo();
+  // The tag containers are removed in async and take some time
+  let oldCountTag1 = 0;
+  let oldCountTag2 = 0;
+  let allTags = await bmsvc.fetchTags();
+  for (let i of allTags) {
+    if (i.name == TAG_1) {
+      oldCountTag1 = i.count;
+    }
+    if (i.name == TAG_2) {
+      oldCountTag2 = i.count;
+    }
+  }
+  await TestUtils.waitForCondition(async () => {
+    allTags = await bmsvc.fetchTags();
+    let newCountTag1 = 0;
+    let newCountTag2 = 0;
+    for (let i of allTags) {
+      if (i.name == TAG_1) {
+        newCountTag1 = i.count;
+      }
+      if (i.name == TAG_2) {
+        newCountTag2 = i.count;
+      }
+    }
+    return newCountTag1 == oldCountTag1 - 1 && newCountTag2 == oldCountTag2 - 1;
+  });
   await ensureItemsRemoved(b2_info);
+
   ensureTags([]);
 
   observer.reset();
@@ -1353,10 +1394,10 @@ add_task(async function test_edit_title() {
   };
 
   function ensureTitleChange(aCurrentTitle) {
-    ensureItemsChanged({
+    ensureItemsTitleChanged({
       guid: bm_info.guid,
-      property: "title",
-      newValue: aCurrentTitle,
+      title: aCurrentTitle,
+      parentGuid: PlacesUtils.bookmarks.unfiledGuid,
     });
   }
 
@@ -1398,10 +1439,9 @@ add_task(async function test_edit_url() {
     aPostChangeURI,
     aOLdURITagsPreserved
   ) {
-    ensureItemsChanged({
+    ensureItemsUrlChanged({
       guid: bm_info.guid,
-      property: "uri",
-      newValue: aPostChangeURI,
+      url: aPostChangeURI,
     });
     ensureTagsForURI(aPostChangeURI, bm_info.tags);
     ensureTagsForURI(aPreChangeURI, aOLdURITagsPreserved ? bm_info.tags : []);
@@ -1758,10 +1798,9 @@ add_task(async function test_untag_uri() {
     }
     function ensureTagsUnset() {
       for (let url of urls) {
-        let expectedTags =
-          tagsRemoved.length == 0
-            ? []
-            : preRemovalTags.get(url).filter(tag => !tagsRemoved.includes(tag));
+        let expectedTags = !tagsRemoved.length
+          ? []
+          : preRemovalTags.get(url).filter(tag => !tagsRemoved.includes(tag));
         ensureTagsForURI(url, expectedTags);
       }
     }
@@ -1973,11 +2012,11 @@ add_task(async function test_invalid_uri_spec_throws() {
   );
   Assert.throws(
     () => PT.Tag({ tag: "TheTag", urls: ["invalid uri spec"] }),
-    /TypeError: invalid uri spec is not a valid URL/
+    /TypeError: URL constructor: invalid uri spec is not a valid URL/
   );
   Assert.throws(
     () => PT.Tag({ tag: "TheTag", urls: ["about:blank", "invalid uri spec"] }),
-    /TypeError: invalid uri spec is not a valid URL/
+    /TypeError: URL constructor: invalid uri spec is not a valid URL/
   );
 });
 
@@ -2133,3 +2172,69 @@ add_task(async function test_renameTag() {
   ensureUndoState();
   await PlacesUtils.bookmarks.eraseEverything();
 });
+
+add_task(async function test_remove_invalid_url() {
+  let folderGuid = await PT.NewFolder({
+    title: "Test Folder",
+    parentGuid: menuGuid,
+  }).transact();
+
+  let guid = "invalid_____";
+  let folderedGuid = "invalid____2";
+  let url = "invalid-uri";
+  await PlacesUtils.withConnectionWrapper("test_bookmarks_remove", async db => {
+    await db.execute(
+      `
+      INSERT INTO moz_places(url, url_hash, title, rev_host, guid)
+      VALUES (:url, hash(:url), 'Invalid URI', '.', GENERATE_GUID())
+    `,
+      { url }
+    );
+    await db.execute(
+      `INSERT INTO moz_bookmarks (type, fk, parent, position, guid)
+        VALUES (:type,
+                (SELECT id FROM moz_places WHERE url = :url),
+                (SELECT id FROM moz_bookmarks WHERE guid = :parentGuid),
+                (SELECT MAX(position) + 1 FROM moz_bookmarks WHERE parent = (SELECT id FROM moz_bookmarks WHERE guid = :parentGuid)),
+                :guid)
+      `,
+      {
+        type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+        url,
+        parentGuid: PlacesUtils.bookmarks.unfiledGuid,
+        guid,
+      }
+    );
+    await db.execute(
+      `INSERT INTO moz_bookmarks (type, fk, parent, position, guid)
+        VALUES (:type,
+                (SELECT id FROM moz_places WHERE url = :url),
+                (SELECT id FROM moz_bookmarks WHERE guid = :parentGuid),
+                (SELECT MAX(position) + 1 FROM moz_bookmarks WHERE parent = (SELECT id FROM moz_bookmarks WHERE guid = :parentGuid)),
+                :guid)
+      `,
+      {
+        type: PlacesUtils.bookmarks.TYPE_BOOKMARK,
+        url,
+        parentGuid: folderGuid,
+        guid: folderedGuid,
+      }
+    );
+  });
+
+  let guids = [folderGuid, guid];
+  await PT.Remove(guids).transact();
+  await ensureNonExistent(...guids, folderedGuid);
+  // Shouldn't throw, should restore the folder but not the bookmarks.
+  await PT.undo();
+  await ensureNonExistent(guid, folderedGuid);
+  Assert.ok(
+    await PlacesUtils.bookmarks.fetch(folderGuid),
+    "The folder should have been re-created"
+  );
+  await PT.redo();
+  await ensureNonExistent(guids, folderedGuid);
+  // Cleanup
+  await PT.clearTransactionsHistory();
+  observer.reset();
+});
