@@ -7,7 +7,9 @@
 
 "use strict";
 
-ChromeUtils.import("resource://gre/modules/SearchStaticData.jsm", this);
+const { SearchStaticData } = ChromeUtils.import(
+  "resource://gre/modules/SearchStaticData.jsm"
+);
 
 function run_test() {
   Assert.ok(
@@ -25,7 +27,7 @@ function run_test() {
       d.startsWith("www.google.")
     )
   );
-  Assert.ok(SearchStaticData.getAlternateDomains("google.com").length == 0);
+  Assert.ok(!SearchStaticData.getAlternateDomains("google.com").length);
 
   // Test that methods from SearchStaticData module can be overwritten,
   // needed for hotfixing.
