@@ -8,6 +8,9 @@
 
 "use strict";
 
+const { AppConstants } = ChromeUtils.import(
+  "resource://gre/modules/AppConstants.jsm"
+);
 const { FormHistory } = ChromeUtils.import(
   "resource://gre/modules/FormHistory.jsm"
 );
@@ -17,6 +20,12 @@ const { SearchSuggestionController } = ChromeUtils.import(
 const { PromiseUtils } = ChromeUtils.import(
   "resource://gre/modules/PromiseUtils.jsm"
 );
+const { TelemetryTestUtils } = ChromeUtils.import(
+  "resource://testing-common/TelemetryTestUtils.jsm"
+);
+
+const ENGINE_NAME = "other";
+const SEARCH_TELEMETRY_LATENCY = "SEARCH_SUGGESTIONS_LATENCY_MS";
 
 // We must make sure the FormHistoryStartup component is
 // initialized in order for it to respond to FormHistory
@@ -26,28 +35,21 @@ var formHistoryStartup = Cc[
 ].getService(Ci.nsIObserver);
 formHistoryStartup.observe(null, "profile-after-change", null);
 
-var httpServer = new HttpServer();
 var getEngine, postEngine, unresolvableEngine, alternateJSONEngine;
 
-function run_test() {
+add_task(async function setup() {
   Services.prefs.setBoolPref("browser.search.suggest.enabled", true);
 
   let server = useHttpServer();
   server.registerContentType("sjs", "sjs");
 
-  registerCleanupFunction(() =>
-    (async function cleanup() {
-      // Remove added form history entries
-      await updateSearchHistory("remove", null);
-      Services.prefs.clearUserPref("browser.search.suggest.enabled");
-    })()
-  );
-
-  run_next_test();
-}
-
-add_task(async function setup() {
   await AddonTestUtils.promiseStartupManager();
+
+  registerCleanupFunction(async () => {
+    // Remove added form history entries
+    await updateSearchHistory("remove", null);
+    Services.prefs.clearUserPref("browser.search.suggest.enabled");
+  });
 });
 
 add_task(async function add_test_engines() {
@@ -76,35 +78,29 @@ add_task(async function add_test_engines() {
     alternativeJSONType: true,
   };
 
-  [
-    getEngine,
-    postEngine,
-    unresolvableEngine,
-    alternateJSONEngine,
-  ] = await addTestEngines([
-    {
-      name: getEngineData.name,
-      xmlFileName: "engineMaker.sjs?" + JSON.stringify(getEngineData),
-    },
-    {
-      name: postEngineData.name,
-      xmlFileName: "engineMaker.sjs?" + JSON.stringify(postEngineData),
-    },
-    {
-      name: unresolvableEngineData.name,
-      xmlFileName: "engineMaker.sjs?" + JSON.stringify(unresolvableEngineData),
-    },
-    {
-      name: alternateJSONSuggestEngineData.name,
-      xmlFileName:
-        "engineMaker.sjs?" + JSON.stringify(alternateJSONSuggestEngineData),
-    },
-  ]);
+  getEngine = await SearchTestUtils.promiseNewSearchEngine(
+    `${gDataUrl}engineMaker.sjs?${JSON.stringify(getEngineData)}`
+  );
+  postEngine = await SearchTestUtils.promiseNewSearchEngine(
+    `${gDataUrl}engineMaker.sjs?${JSON.stringify(postEngineData)}`
+  );
+  unresolvableEngine = await SearchTestUtils.promiseNewSearchEngine(
+    `${gDataUrl}engineMaker.sjs?${JSON.stringify(unresolvableEngineData)}`
+  );
+  alternateJSONEngine = await SearchTestUtils.promiseNewSearchEngine(
+    `${gDataUrl}engineMaker.sjs?${JSON.stringify(
+      alternateJSONSuggestEngineData
+    )}`
+  );
 });
 
 // Begin tests
 
 add_task(async function simple_no_result_callback() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   await new Promise(resolve => {
     let controller = new SearchSuggestionController(result => {
       Assert.equal(result.term, "no remote");
@@ -115,9 +111,15 @@ add_task(async function simple_no_result_callback() {
 
     controller.fetch("no remote", false, getEngine);
   });
+
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function simple_no_result_callback_and_promise() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   // Make sure both the callback and promise get results
   let deferred = PromiseUtils.defer();
   let controller = new SearchSuggestionController(result => {
@@ -133,25 +135,65 @@ add_task(async function simple_no_result_callback_and_promise() {
   Assert.equal(result.remote.length, 0);
 
   await deferred.promise;
+
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function simple_no_result_promise() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   let result = await controller.fetch("no remote", false, getEngine);
   Assert.equal(result.term, "no remote");
   Assert.equal(result.local.length, 0);
   Assert.equal(result.remote.length, 0);
+
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function simple_remote_no_local_result() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   let result = await controller.fetch("mo", false, getEngine);
   Assert.equal(result.term, "mo");
   Assert.equal(result.local.length, 0);
   Assert.equal(result.remote.length, 3);
-  Assert.equal(result.remote[0], "Mozilla");
-  Assert.equal(result.remote[1], "modern");
-  Assert.equal(result.remote[2], "mom");
+  Assert.equal(result.remote[0].value, "Mozilla");
+  Assert.equal(result.remote[1].value, "modern");
+  Assert.equal(result.remote[2].value, "mom");
+
+  assertLatencyHistogram(histogram, true);
+});
+
+add_task(async function simple_remote_no_local_result_telemetry() {
+  Services.telemetry.clearScalars();
+
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
+  let controller = new SearchSuggestionController();
+  await controller.fetch("mo", false, getEngine);
+
+  let scalars = {};
+  const key = "browser.search.data_transferred";
+
+  await TestUtils.waitForCondition(() => {
+    scalars =
+      Services.telemetry.getSnapshotForKeyedScalars("main", false).parent || {};
+    return key in scalars;
+  }, "should have the expected keyed scalars");
+
+  const scalar = scalars[key];
+  Assert.ok(`sggt-${ENGINE_NAME}` in scalar, "correct telemetry category");
+  Assert.notEqual(scalar[`sggt-${ENGINE_NAME}`], 0, "bandwidth logged");
+
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function simple_remote_no_local_result_alternative_type() {
@@ -160,9 +202,9 @@ add_task(async function simple_remote_no_local_result_alternative_type() {
   Assert.equal(result.term, "mo");
   Assert.equal(result.local.length, 0);
   Assert.equal(result.remote.length, 3);
-  Assert.equal(result.remote[0], "Mozilla");
-  Assert.equal(result.remote[1], "modern");
-  Assert.equal(result.remote[2], "mom");
+  Assert.equal(result.remote[0].value, "Mozilla");
+  Assert.equal(result.remote[1].value, "modern");
+  Assert.equal(result.remote[2].value, "mom");
 });
 
 add_task(async function remote_term_case_mismatch() {
@@ -170,7 +212,7 @@ add_task(async function remote_term_case_mismatch() {
   let result = await controller.fetch("Query Case Mismatch", false, getEngine);
   Assert.equal(result.term, "Query Case Mismatch");
   Assert.equal(result.remote.length, 1);
-  Assert.equal(result.remote[0], "Query Case Mismatch");
+  Assert.equal(result.remote[0].value, "Query Case Mismatch");
 });
 
 add_task(async function simple_local_no_remote_result() {
@@ -180,7 +222,7 @@ add_task(async function simple_local_no_remote_result() {
   let result = await controller.fetch("no remote", false, getEngine);
   Assert.equal(result.term, "no remote");
   Assert.equal(result.local.length, 1);
-  Assert.equal(result.local[0], "no remote entries");
+  Assert.equal(result.local[0].value, "no remote entries");
   Assert.equal(result.remote.length, 0);
 
   await updateSearchHistory("remove", "no remote entries");
@@ -193,9 +235,9 @@ add_task(async function simple_non_ascii() {
   let result = await controller.fetch("I ❤️", false, getEngine);
   Assert.equal(result.term, "I ❤️");
   Assert.equal(result.local.length, 1);
-  Assert.equal(result.local[0], "I ❤️ XUL");
+  Assert.equal(result.local[0].value, "I ❤️ XUL");
   Assert.equal(result.remote.length, 1);
-  Assert.equal(result.remote[0], "I ❤️ Mozilla");
+  Assert.equal(result.remote[0].value, "I ❤️ Mozilla");
 });
 
 add_task(async function both_local_remote_result_dedupe() {
@@ -205,10 +247,10 @@ add_task(async function both_local_remote_result_dedupe() {
   let result = await controller.fetch("mo", false, getEngine);
   Assert.equal(result.term, "mo");
   Assert.equal(result.local.length, 1);
-  Assert.equal(result.local[0], "Mozilla");
+  Assert.equal(result.local[0].value, "Mozilla");
   Assert.equal(result.remote.length, 2);
-  Assert.equal(result.remote[0], "modern");
-  Assert.equal(result.remote[1], "mom");
+  Assert.equal(result.remote[0].value, "modern");
+  Assert.equal(result.remote[1].value, "mom");
 });
 
 add_task(async function POST_both_local_remote_result_dedupe() {
@@ -216,10 +258,10 @@ add_task(async function POST_both_local_remote_result_dedupe() {
   let result = await controller.fetch("mo", false, postEngine);
   Assert.equal(result.term, "mo");
   Assert.equal(result.local.length, 1);
-  Assert.equal(result.local[0], "Mozilla");
+  Assert.equal(result.local[0].value, "Mozilla");
   Assert.equal(result.remote.length, 2);
-  Assert.equal(result.remote[0], "modern");
-  Assert.equal(result.remote[1], "mom");
+  Assert.equal(result.remote[0].value, "modern");
+  Assert.equal(result.remote[1].value, "mom");
 });
 
 add_task(async function both_local_remote_result_dedupe2() {
@@ -229,10 +271,10 @@ add_task(async function both_local_remote_result_dedupe2() {
   let result = await controller.fetch("mo", false, getEngine);
   Assert.equal(result.term, "mo");
   Assert.equal(result.local.length, 2);
-  Assert.equal(result.local[0], "mom");
-  Assert.equal(result.local[1], "Mozilla");
+  Assert.equal(result.local[0].value, "mom");
+  Assert.equal(result.local[1].value, "Mozilla");
   Assert.equal(result.remote.length, 1);
-  Assert.equal(result.remote[0], "modern");
+  Assert.equal(result.remote[0].value, "modern");
 });
 
 add_task(async function both_local_remote_result_dedupe3() {
@@ -243,13 +285,114 @@ add_task(async function both_local_remote_result_dedupe3() {
   let result = await controller.fetch("mo", false, getEngine);
   Assert.equal(result.term, "mo");
   Assert.equal(result.local.length, 3);
-  Assert.equal(result.local[0], "modern");
-  Assert.equal(result.local[1], "mom");
-  Assert.equal(result.local[2], "Mozilla");
+  Assert.equal(result.local[0].value, "modern");
+  Assert.equal(result.local[1].value, "mom");
+  Assert.equal(result.local[2].value, "Mozilla");
   Assert.equal(result.remote.length, 0);
 });
 
+add_task(async function valid_tail_results() {
+  let controller = new SearchSuggestionController();
+  let result = await controller.fetch("tail query", false, getEngine);
+  Assert.equal(result.term, "tail query");
+  Assert.equal(result.local.length, 0);
+  Assert.equal(result.remote.length, 3);
+  Assert.equal(result.remote[0].value, "tail query normal");
+  Assert.ok(!result.remote[0].matchPrefix);
+  Assert.ok(!result.remote[0].tail);
+  Assert.equal(result.remote[1].value, "tail query tail 1");
+  Assert.equal(result.remote[1].matchPrefix, "… ");
+  Assert.equal(result.remote[1].tail, "tail 1");
+  Assert.equal(result.remote[2].value, "tail query tail 2");
+  Assert.equal(result.remote[2].matchPrefix, "… ");
+  Assert.equal(result.remote[2].tail, "tail 2");
+});
+
+add_task(async function alt_tail_results() {
+  let controller = new SearchSuggestionController();
+  let result = await controller.fetch("tailalt query", false, getEngine);
+  Assert.equal(result.term, "tailalt query");
+  Assert.equal(result.local.length, 0);
+  Assert.equal(result.remote.length, 3);
+  Assert.equal(result.remote[0].value, "tailalt query normal");
+  Assert.ok(!result.remote[0].matchPrefix);
+  Assert.ok(!result.remote[0].tail);
+  Assert.equal(result.remote[1].value, "tailalt query tail 1");
+  Assert.equal(result.remote[1].matchPrefix, "… ");
+  Assert.equal(result.remote[1].tail, "tail 1");
+  Assert.equal(result.remote[2].value, "tailalt query tail 2");
+  Assert.equal(result.remote[2].matchPrefix, "… ");
+  Assert.equal(result.remote[2].tail, "tail 2");
+});
+
+add_task(async function invalid_tail_results() {
+  let controller = new SearchSuggestionController();
+  let result = await controller.fetch("tailjunk query", false, getEngine);
+  Assert.equal(result.term, "tailjunk query");
+  Assert.equal(result.local.length, 0);
+  Assert.equal(result.remote.length, 3);
+  Assert.equal(result.remote[0].value, "tailjunk query normal");
+  Assert.ok(!result.remote[0].matchPrefix);
+  Assert.ok(!result.remote[0].tail);
+  Assert.equal(result.remote[1].value, "tailjunk query tail 1");
+  Assert.ok(!result.remote[1].matchPrefix);
+  Assert.ok(!result.remote[1].tail);
+  Assert.equal(result.remote[2].value, "tailjunk query tail 2");
+  Assert.ok(!result.remote[2].matchPrefix);
+  Assert.ok(!result.remote[2].tail);
+});
+
+add_task(async function too_few_tail_results() {
+  let controller = new SearchSuggestionController();
+  let result = await controller.fetch("tailjunk few query", false, getEngine);
+  Assert.equal(result.term, "tailjunk few query");
+  Assert.equal(result.local.length, 0);
+  Assert.equal(result.remote.length, 3);
+  Assert.equal(result.remote[0].value, "tailjunk few query normal");
+  Assert.ok(!result.remote[0].matchPrefix);
+  Assert.ok(!result.remote[0].tail);
+  Assert.equal(result.remote[1].value, "tailjunk few query tail 1");
+  Assert.ok(!result.remote[1].matchPrefix);
+  Assert.ok(!result.remote[1].tail);
+  Assert.equal(result.remote[2].value, "tailjunk few query tail 2");
+  Assert.ok(!result.remote[2].matchPrefix);
+  Assert.ok(!result.remote[2].tail);
+});
+
+add_task(async function empty_rich_results() {
+  let controller = new SearchSuggestionController();
+  let result = await controller.fetch("richempty query", false, getEngine);
+  Assert.equal(result.term, "richempty query");
+  Assert.equal(result.local.length, 0);
+  Assert.equal(result.remote.length, 3);
+  Assert.equal(result.remote[0].value, "richempty query normal");
+  Assert.ok(!result.remote[0].matchPrefix);
+  Assert.ok(!result.remote[0].tail);
+  Assert.equal(result.remote[1].value, "richempty query tail 1");
+  Assert.ok(!result.remote[1].matchPrefix);
+  Assert.ok(!result.remote[1].tail);
+  Assert.equal(result.remote[2].value, "richempty query tail 2");
+  Assert.ok(!result.remote[2].matchPrefix);
+  Assert.ok(!result.remote[2].tail);
+});
+
+add_task(async function tail_offset_index() {
+  let controller = new SearchSuggestionController();
+  let result = await controller.fetch("tail tail 1 t", false, getEngine);
+  Assert.equal(result.term, "tail tail 1 t");
+  Assert.equal(result.local.length, 0);
+  Assert.equal(result.remote.length, 3);
+  Assert.equal(result.remote[1].value, "tail tail 1 t tail 1");
+  Assert.equal(result.remote[1].matchPrefix, "… ");
+  Assert.equal(result.remote[1].tail, "tail 1");
+  Assert.equal(result.remote[1].tailOffsetIndex, 14);
+});
+
 add_task(async function fetch_twice_in_a_row() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   // Two entries since the first will match the first fetch but not the second.
   await updateSearchHistory("bump", "delay local");
   await updateSearchHistory("bump", "delayed local");
@@ -264,9 +407,13 @@ add_task(async function fetch_twice_in_a_row() {
   let result = await resultPromise2;
   Assert.equal(result.term, "delayed ");
   Assert.equal(result.local.length, 1);
-  Assert.equal(result.local[0], "delayed local");
+  Assert.equal(result.local[0].value, "delayed local");
   Assert.equal(result.remote.length, 1);
-  Assert.equal(result.remote[0], "delayed ");
+  Assert.equal(result.remote[0].value, "delayed ");
+
+  // Only the second fetch's latency should be recorded since the first fetch
+  // was aborted and latencies for aborted fetches are not recorded.
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function fetch_twice_subset_reuse_formHistoryResult() {
@@ -279,10 +426,10 @@ add_task(async function fetch_twice_subset_reuse_formHistoryResult() {
   let result = await controller.fetch("delay", false, getEngine);
   Assert.equal(result.term, "delay");
   Assert.equal(result.local.length, 2);
-  Assert.equal(result.local[0], "delay local");
-  Assert.equal(result.local[1], "delayed local");
+  Assert.equal(result.local[0].value, "delay local");
+  Assert.equal(result.local[1].value, "delayed local");
   Assert.equal(result.remote.length, 1);
-  Assert.equal(result.remote[0], "delay");
+  Assert.equal(result.remote[0].value, "delay");
 
   // Remove the entry from the DB but it should remain in the cached formHistoryResult.
   await updateSearchHistory("remove", "delayed local");
@@ -290,9 +437,9 @@ add_task(async function fetch_twice_subset_reuse_formHistoryResult() {
   let result2 = await controller.fetch("delayed ", false, getEngine);
   Assert.equal(result2.term, "delayed ");
   Assert.equal(result2.local.length, 1);
-  Assert.equal(result2.local[0], "delayed local");
+  Assert.equal(result2.local[0].value, "delayed local");
   Assert.equal(result2.remote.length, 1);
-  Assert.equal(result2.remote[0], "delayed ");
+  Assert.equal(result2.remote[0].value, "delayed ");
 });
 
 add_task(async function both_identical_with_more_than_max_results() {
@@ -316,14 +463,14 @@ add_task(async function both_identical_with_more_than_max_results() {
   Assert.equal(result.local.length, 7);
   for (let i = 0; i < controller.maxLocalResults; i++) {
     Assert.equal(
-      result.local[i],
+      result.local[i].value,
       "letter " + String.fromCharCode("A".charCodeAt() + i)
     );
   }
   Assert.equal(result.local.length + result.remote.length, 10);
   for (let i = 0; i < result.remote.length; i++) {
     Assert.equal(
-      result.remote[i],
+      result.remote[i].value,
       "letter " +
         String.fromCharCode("A".charCodeAt() + controller.maxLocalResults + i)
     );
@@ -331,6 +478,10 @@ add_task(async function both_identical_with_more_than_max_results() {
 });
 
 add_task(async function noremote_maxLocal() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   controller.maxLocalResults = 2; // (should be ignored because no remote results)
   controller.maxRemoteResults = 0;
@@ -339,14 +490,20 @@ add_task(async function noremote_maxLocal() {
   Assert.equal(result.local.length, 26);
   for (let i = 0; i < result.local.length; i++) {
     Assert.equal(
-      result.local[i],
+      result.local[i].value,
       "letter " + String.fromCharCode("A".charCodeAt() + i)
     );
   }
   Assert.equal(result.remote.length, 0);
+
+  assertLatencyHistogram(histogram, false);
 });
 
 add_task(async function someremote_maxLocal() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   controller.maxLocalResults = 2;
   controller.maxRemoteResults = 4;
@@ -355,7 +512,7 @@ add_task(async function someremote_maxLocal() {
   Assert.equal(result.local.length, 2);
   for (let i = 0; i < result.local.length; i++) {
     Assert.equal(
-      result.local[i],
+      result.local[i].value,
       "letter " + String.fromCharCode("A".charCodeAt() + i)
     );
   }
@@ -363,10 +520,12 @@ add_task(async function someremote_maxLocal() {
   // "A" and "B" will have been de-duped, start at C for remote results
   for (let i = 0; i < result.remote.length; i++) {
     Assert.equal(
-      result.remote[i],
+      result.remote[i].value,
       "letter " + String.fromCharCode("C".charCodeAt() + i)
     );
   }
+
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function one_of_each() {
@@ -376,12 +535,16 @@ add_task(async function one_of_each() {
   let result = await controller.fetch("letter ", false, getEngine);
   Assert.equal(result.term, "letter ");
   Assert.equal(result.local.length, 1);
-  Assert.equal(result.local[0], "letter A");
+  Assert.equal(result.local[0].value, "letter A");
   Assert.equal(result.remote.length, 1);
-  Assert.equal(result.remote[0], "letter B");
+  Assert.equal(result.remote[0].value, "letter B");
 });
 
 add_task(async function local_result_returned_remote_result_disabled() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   Services.prefs.setBoolPref("browser.search.suggest.enabled", false);
   let controller = new SearchSuggestionController();
   controller.maxLocalResults = 1;
@@ -391,16 +554,20 @@ add_task(async function local_result_returned_remote_result_disabled() {
   Assert.equal(result.local.length, 26);
   for (let i = 0; i < 26; i++) {
     Assert.equal(
-      result.local[i],
+      result.local[i].value,
       "letter " + String.fromCharCode("A".charCodeAt() + i)
     );
   }
   Assert.equal(result.remote.length, 0);
+  assertLatencyHistogram(histogram, false);
   Services.prefs.setBoolPref("browser.search.suggest.enabled", true);
 });
 
 add_task(
   async function local_result_returned_remote_result_disabled_after_creation_of_controller() {
+    let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+      SEARCH_TELEMETRY_LATENCY
+    );
     let controller = new SearchSuggestionController();
     controller.maxLocalResults = 1;
     controller.maxRemoteResults = 1;
@@ -410,17 +577,22 @@ add_task(
     Assert.equal(result.local.length, 26);
     for (let i = 0; i < 26; i++) {
       Assert.equal(
-        result.local[i],
+        result.local[i].value,
         "letter " + String.fromCharCode("A".charCodeAt() + i)
       );
     }
     Assert.equal(result.remote.length, 0);
+    assertLatencyHistogram(histogram, false);
     Services.prefs.setBoolPref("browser.search.suggest.enabled", true);
   }
 );
 
 add_task(
   async function one_of_each_disabled_before_creation_enabled_after_creation_of_controller() {
+    let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+      SEARCH_TELEMETRY_LATENCY
+    );
+
     Services.prefs.setBoolPref("browser.search.suggest.enabled", false);
     let controller = new SearchSuggestionController();
     controller.maxLocalResults = 1;
@@ -429,17 +601,20 @@ add_task(
     let result = await controller.fetch("letter ", false, getEngine);
     Assert.equal(result.term, "letter ");
     Assert.equal(result.local.length, 1);
-    Assert.equal(result.local[0], "letter A");
+    Assert.equal(result.local[0].value, "letter A");
     Assert.equal(result.remote.length, 1);
-    Assert.equal(result.remote[0], "letter B");
+    Assert.equal(result.remote[0].value, "letter B");
+
+    assertLatencyHistogram(histogram, true);
+
+    Services.prefs.setBoolPref("browser.search.suggest.enabled", true);
   }
 );
 
-add_task(async function reset_suggestions_pref() {
-  Services.prefs.setBoolPref("browser.search.suggest.enabled", true);
-});
-
 add_task(async function one_local_zero_remote() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
   let controller = new SearchSuggestionController();
   controller.maxLocalResults = 1;
   controller.maxRemoteResults = 0;
@@ -448,14 +623,18 @@ add_task(async function one_local_zero_remote() {
   Assert.equal(result.local.length, 26);
   for (let i = 0; i < 26; i++) {
     Assert.equal(
-      result.local[i],
+      result.local[i].value,
       "letter " + String.fromCharCode("A".charCodeAt() + i)
     );
   }
   Assert.equal(result.remote.length, 0);
+  assertLatencyHistogram(histogram, false);
 });
 
 add_task(async function zero_local_one_remote() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
   let controller = new SearchSuggestionController();
   controller.maxLocalResults = 0;
   controller.maxRemoteResults = 1;
@@ -463,10 +642,14 @@ add_task(async function zero_local_one_remote() {
   Assert.equal(result.term, "letter ");
   Assert.equal(result.local.length, 0);
   Assert.equal(result.remote.length, 1);
-  Assert.equal(result.remote[0], "letter A");
+  Assert.equal(result.remote[0].value, "letter A");
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function stop_search() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
   let controller = new SearchSuggestionController(result => {
     do_throw("The callback shouldn't be called after stop()");
   });
@@ -475,52 +658,122 @@ add_task(async function stop_search() {
   await resultPromise.then(result => {
     Assert.equal(null, result);
   });
+  assertLatencyHistogram(histogram, false);
 });
 
 add_task(async function empty_searchTerm() {
   // Empty searches don't go to the server but still get form history.
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
   let controller = new SearchSuggestionController();
   let result = await controller.fetch("", false, getEngine);
   Assert.equal(result.term, "");
-  Assert.ok(result.local.length > 0);
+  Assert.ok(!!result.local.length);
   Assert.equal(result.remote.length, 0);
+  assertLatencyHistogram(histogram, false);
 });
 
 add_task(async function slow_timeout() {
-  let d = PromiseUtils.defer();
-  function check_result(result) {
-    Assert.equal(result.term, "slow ");
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
+  // Make the server return suggestions on a delay longer than the timeout of
+  // the suggestion controller.
+  let delayMs = 3 * SearchSuggestionController.REMOTE_TIMEOUT_DEFAULT;
+  let searchString = `delay${delayMs} `;
+
+  // Add a local result.
+  let localValue = searchString + " local result";
+  await updateSearchHistory("bump", localValue);
+
+  // Do a search. The remote fetch should time out but the local result should
+  // be returned.
+  let controller = new SearchSuggestionController();
+  let result = await controller.fetch(searchString, false, getEngine);
+  Assert.equal(result.term, searchString);
+  Assert.equal(result.local.length, 1);
+  Assert.equal(result.local[0].value, localValue);
+  Assert.equal(result.remote.length, 0);
+
+  // The remote fetch isn't done yet, so the latency histogram should not be
+  // updated.
+  assertLatencyHistogram(histogram, false);
+
+  // Wait for the remote fetch to finish.
+  await new Promise(r => setTimeout(r, delayMs));
+
+  // Now the latency histogram should be updated.
+  assertLatencyHistogram(histogram, true);
+});
+
+add_task(async function slow_timeout_2() {
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
+  // Make the server return suggestions on a delay longer the timeout of the
+  // suggestion controller.
+  let delayMs = 3 * SearchSuggestionController.REMOTE_TIMEOUT_DEFAULT;
+  let searchString = `delay${delayMs} `;
+
+  // Add a local result.
+  let localValue = searchString + " local result";
+  await updateSearchHistory("bump", localValue);
+
+  // Do two searches using the same controller. Both times, the remote fetches
+  // should time out and only the local result should be returned. The second
+  // search should abort the remote fetch of the first search, and the remote
+  // fetch of the second search should be ongoing when the second search
+  // finishes.
+  let controller = new SearchSuggestionController();
+  for (let i = 0; i < 2; i++) {
+    let result = await controller.fetch(searchString, false, getEngine);
+    Assert.equal(result.term, searchString);
     Assert.equal(result.local.length, 1);
-    Assert.equal(result.local[0], "slow local result");
+    Assert.equal(result.local[0].value, localValue);
     Assert.equal(result.remote.length, 0);
   }
-  await updateSearchHistory("bump", "slow local result");
 
-  let controller = new SearchSuggestionController();
-  setTimeout(function check_timeout() {
-    // The HTTP response takes 10 seconds so check that we already have results after 2 seconds.
-    check_result(result);
-    d.resolve();
-  }, 2000);
-  let result = await controller.fetch("slow ", false, getEngine);
-  check_result(result);
-  await d.promise;
+  // The remote fetch of the second search isn't done yet, so the latency
+  // histogram should not be updated.
+  assertLatencyHistogram(histogram, false);
+
+  // Wait for the second remote fetch to finish.
+  await new Promise(r => setTimeout(r, delayMs));
+
+  // Now the latency histogram should be updated, and only the remote fetch of
+  // the second search should be recorded.
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function slow_stop() {
-  let d = PromiseUtils.defer();
-  let controller = new SearchSuggestionController();
-  let resultPromise = controller.fetch("slow ", false, getEngine);
-  setTimeout(function check_timeout() {
-    // The HTTP response takes 10 seconds but we timeout in less than a second so just use 0.
-    controller.stop();
-    d.resolve();
-  }, 0);
-  await resultPromise.then(result => {
-    Assert.equal(null, result);
-  });
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
 
-  await d.promise;
+  // Make the server return suggestions on a delay longer the timeout of the
+  // suggestion controller.
+  let delayMs = 3 * SearchSuggestionController.REMOTE_TIMEOUT_DEFAULT;
+  let searchString = `delay${delayMs} `;
+
+  // Do a search but stop it before it finishes. Wait a tick before stopping it
+  // to better simulate the real world.
+  let controller = new SearchSuggestionController();
+  let resultPromise = controller.fetch(searchString, false, getEngine);
+  await TestUtils.waitForTick();
+  controller.stop();
+  let result = await resultPromise;
+  Assert.equal(result, null, "No result should be returned");
+
+  // The remote fetch should have been aborted by stopping the controller, but
+  // wait for the timeout period just to make sure it's done.
+  await new Promise(r => setTimeout(r, delayMs));
+
+  // Since the latencies of aborted fetches are not recorded, the latency
+  // histogram should not be updated.
+  assertLatencyHistogram(histogram, false);
 });
 
 // Error handling
@@ -528,39 +781,61 @@ add_task(async function slow_stop() {
 add_task(async function remote_term_mismatch() {
   await updateSearchHistory("bump", "Query Mismatch Entry");
 
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   let result = await controller.fetch("Query Mismatch", false, getEngine);
   Assert.equal(result.term, "Query Mismatch");
   Assert.equal(result.local.length, 1);
-  Assert.equal(result.local[0], "Query Mismatch Entry");
+  Assert.equal(result.local[0].value, "Query Mismatch Entry");
   Assert.equal(result.remote.length, 0);
+
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function http_404() {
   await updateSearchHistory("bump", "HTTP 404 Entry");
 
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   let result = await controller.fetch("HTTP 404", false, getEngine);
   Assert.equal(result.term, "HTTP 404");
   Assert.equal(result.local.length, 1);
-  Assert.equal(result.local[0], "HTTP 404 Entry");
+  Assert.equal(result.local[0].value, "HTTP 404 Entry");
   Assert.equal(result.remote.length, 0);
+
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function http_500() {
   await updateSearchHistory("bump", "HTTP 500 Entry");
 
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   let result = await controller.fetch("HTTP 500", false, getEngine);
   Assert.equal(result.term, "HTTP 500");
   Assert.equal(result.local.length, 1);
-  Assert.equal(result.local[0], "HTTP 500 Entry");
+  Assert.equal(result.local[0].value, "HTTP 500 Entry");
   Assert.equal(result.remote.length, 0);
+
+  assertLatencyHistogram(histogram, true);
 });
 
 add_task(async function unresolvable_server() {
   await updateSearchHistory("bump", "Unresolvable Server Entry");
 
+  let histogram = TelemetryTestUtils.getAndClearKeyedHistogram(
+    SEARCH_TELEMETRY_LATENCY
+  );
+
   let controller = new SearchSuggestionController();
   let result = await controller.fetch(
     "Unresolvable Server",
@@ -569,8 +844,13 @@ add_task(async function unresolvable_server() {
   );
   Assert.equal(result.term, "Unresolvable Server");
   Assert.equal(result.local.length, 1);
-  Assert.equal(result.local[0], "Unresolvable Server Entry");
+  Assert.equal(result.local[0].value, "Unresolvable Server Entry");
   Assert.equal(result.remote.length, 0);
+
+  // This latency assert fails on Windows 7 (NT version 6.1), so skip it there.
+  if (!AppConstants.isPlatformAndVersionAtMost("win", "6.1")) {
+    assertLatencyHistogram(histogram, true);
+  }
 });
 
 // Exception handling
@@ -628,6 +908,18 @@ add_task(async function test_userContextId() {
   controller.fetch("test", false, getEngine, 1);
 });
 
+// Non-English characters
+
+add_task(async function suggestions_contain_escaped_unicode() {
+  let controller = new SearchSuggestionController();
+  let result = await controller.fetch("stü", false, getEngine);
+  Assert.equal(result.term, "stü");
+  Assert.equal(result.local.length, 0);
+  Assert.equal(result.remote.length, 2);
+  Assert.equal(result.remote[0].value, "stühle");
+  Assert.equal(result.remote[1].value, "stüssy");
+});
+
 // Helpers
 
 function updateSearchHistory(operation, value) {
@@ -652,3 +944,24 @@ function updateSearchHistory(operation, value) {
     );
   });
 }
+
+function assertLatencyHistogram(histogram, shouldRecord) {
+  let snapshot = histogram.snapshot();
+  info("Checking latency snapshot: " + JSON.stringify(snapshot));
+
+  // Build a map from engine ID => number of non-zero values recorded for it.
+  let valueCountByEngineId = Object.entries(snapshot).reduce(
+    (memo, [key, data]) => {
+      memo[key] = Object.values(data.values).filter(v => v != 0);
+      return memo;
+    },
+    {}
+  );
+
+  let expected = shouldRecord ? { [ENGINE_NAME]: [1] } : {};
+  Assert.deepEqual(
+    valueCountByEngineId,
+    expected,
+    shouldRecord ? "Latency histogram updated" : "Latency histogram not updated"
+  );
+}
